@@ -1,10 +1,13 @@
-;;; hact.el --- GNU Hyperbole button action handling
+;;; hact.el --- GNU Hyperbole button action handling  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    18-Sep-91 at 02:57:09
+;; Last-Mod:      7-Nov-23 at 21:38:28 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -16,7 +19,18 @@
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
-(eval-and-compile (mapc #'require '(hhist hpath set)))
+(eval-and-compile (mapc #'require '(hhist set)))
+
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function hattr:get "hypb")
+(declare-function hattr:list "hypb")
+(declare-function hattr:set "hypb")
+(declare-function hbut:is-p "hypb")
+(declare-function hpath:absolute-arguments "hpath")
+(declare-function hypb:indirect-function "hypb")
 
 ;;; ************************************************************************
 ;;; Public variables
@@ -36,10 +50,24 @@ e.g. to inhibit actions.")
 ;;; ========================================================================
 
 (defvar symtable:category-plist nil
-  "Holds a property list of Hyperbole type category symbols ('actypes or 'ibtypes) and their associated symtables.")
+  "Property list mapping Hyperbole type category symbols to symtables.
+The type categories are either `actypes' or `ibtypes'.")
 
-(defun  symtable:operate (operation symbol-or-name symtable)
-  "Call hash-table function OPERATION with Hyperbole SYMBOL-OR-NAME as key upon SYMTABLE.
+(defsubst symtable:hash-table (symtable)
+  "Return the hash-table containing symbol names and values from SYMTABLE."
+  (plist-get symtable 'hash-table))
+
+(defsubst symtable:name (symtable)
+  "Return the name of SYMTABLE as a string."
+  (plist-get symtable 'name))
+
+(defsubst symtable:select (type-category)
+  "Inline the return of the symtable for TYPE-CATEGORY.
+TYPE-CATEGORY is one of `actypes' or `ibtypes'."
+  (plist-get symtable:category-plist type-category))
+
+(defun    symtable:operate (operation symbol-or-name symtable)
+  "Call hash-table OPERATION with Hyperbole SYMBOL-OR-NAME key for SYMTABLE.
 Trigger an error if SYMBOL-OR-NAME cannot be mapped to an existing Elisp
 symbol or if SYMTABLE is invalid."
   (let ((name (cond ((stringp symbol-or-name)
@@ -77,10 +105,6 @@ symbol or if SYMTABLE is invalid."
        (gethash def-name  hash-table))
       (_ (error "(symtable:operate): Invalid operation request: %s" operation)))))
 
-(defsubst symtable:select (type-category)
-  "Inline the return of the symtable for TYPE-CATEGORY, one of 'actypes or 'ibtypes."
-  (plist-get symtable:category-plist type-category))
-
 (defun    symtable:create (name size)
   "Create and return a new Hyperbole type symbol table with NAME and SIZE.
 Also add it under the symbol for its NAME in `symtable:category-plist'."
@@ -89,14 +113,6 @@ Also add it under the symbol for its NAME in `symtable:category-plist'."
     (setq symtable:category-plist (plist-put symtable:category-plist (intern name) symtable))
     symtable))
 
-(defsubst symtable:hash-table (symtable)
-  "Return the hash-table containing symbol names and values from SYMTABLE."
-  (plist-get symtable 'hash-table))
-
-(defsubst symtable:name (symtable)
-  "Return the name of SYMTABLE as a string."
-  (plist-get symtable 'name))
-
 (defvar   symtable:actypes (symtable:create "actypes" 97)
   "Symbol table (hash table) of Hyperbole action type symbols.
 For each actype, there are two entries whose keys are strings: one
@@ -110,12 +126,24 @@ with the `ibtypes::' prefix and one without.  The value for both
 keys is the Elisp symbol for the type, which includes the prefix.")
 
 (defsubst symtable:actype-p (symbol-or-name)
-  "Return the Elisp symbol given by SYMBOL-OR-NAME if it is a Hyperbole action type name, else nil."
-  (symtable:get symbol-or-name symtable:actypes))
+  "Return SYMBOL-OR-NAME if a Hyperbole action type or Elisp function, else nil."
+  (when (or (symbolp symbol-or-name) (stringp symbol-or-name))
+    (or (symtable:get symbol-or-name symtable:actypes)
+	(and (stringp symbol-or-name) (fboundp (intern-soft symbol-or-name))
+	     (intern-soft symbol-or-name))
+	(and (functionp symbol-or-name) symbol-or-name))))
+
+(defsubst symtable:hyperbole-actype-p (symbol-or-name)
+  "Return SYMBOL-OR-NAME if a Hyperbole action type, else nil.
+This excludes Emacs Lisp functions which may be used as action types.
+Use `actype:elisp-symbol' to include these."
+  (when (or (symbolp symbol-or-name) (stringp symbol-or-name))
+    (or (symtable:get symbol-or-name symtable:actypes))))
 
 (defsubst symtable:ibtype-p (symbol-or-name)
-  "Return the Elisp symbol given by SYMBOL-OR-NAME if it is a Hyperbole implicit button type name, else nil."
-  (symtable:get symbol-or-name symtable:ibtypes))
+  "Return SYMBOL-OR-NAME if it is a Hyperbole implicit button type, else nil."
+  (when (or (symbolp symbol-or-name) (stringp symbol-or-name))
+    (symtable:get symbol-or-name symtable:ibtypes)))
 
 (defun    symtable:add (symbol-or-name symtable)
   "Add Hyperbole SYMBOL-OR-NAME to existing SYMTABLE.
@@ -123,15 +151,15 @@ Return the Elisp symbol for SYMBOL-OR-NAME.
 Caller must ensure SYMBOL-OR-NAME is a symbol or string."
   (symtable:operate #'puthash symbol-or-name symtable))
 
-(defalias 'symtable:delete 'symtable:remove)
+(defalias 'symtable:delete #'symtable:remove)
 
 (defun    symtable:get (symbol-or-name symtable)
-  "Return the Elisp symbol given by Hyperbole SYMBOL-OR-NAME if it is in existing SYMTABLE, else nil.
+  "Return Hyperbole SYMBOL-OR-NAME if it is in SYMTABLE, else nil.
 Caller must ensure SYMBOL-OR-NAME is a symbol or string."
   (symtable:operate #'gethash symbol-or-name symtable))
 
 (defun    symtable:remove (symbol-or-name symtable)
-  "Remove the Elisp symbol given by Hyperbole SYMBOL-OR-NAME if it is in existing SYMTABLE.
+  "Remove the Hyperbole SYMBOL-OR-NAME if it is in SYMTABLE.
 Always return nil.
 Caller must ensure SYMBOL-OR-NAME is a symbol or string."
   (symtable:operate #'remhash symbol-or-name symtable))
@@ -153,14 +181,19 @@ If no SYMBOLS are given, set it to the empty set.  Return the symset.  Uses
 
 (defun    symset:add (elt symbol property)
   "Add ELT to SYMBOL's PROPERTY set.
-Return nil iff ELT is already in SET; otherwise, return PROPERTY's value.
+Return PROPERTY's value unless ELT is already in the set, then return nil.
 Use `eq' for comparison."
   (let* ((set (get symbol property))
 	 (set:equal-op 'eq)
 	 (new-set (set:add elt set)))
-    (and new-set (put symbol property new-set))))
+    (when new-set
+      (put symbol property new-set))))
 
-(defalias 'symset:delete 'symset:remove)
+(defun    symset:clear (symbol)
+  "Set SYMBOL's symset to nil."
+  (setf (symbol-plist symbol) nil))
+
+(defalias 'symset:delete #'symset:remove)
 
 (defun    symset:get (symbol property)
   "Return SYMBOL's PROPERTY set."
@@ -184,7 +217,7 @@ Assume PROPERTY is a valid set.  Use `eq' for comparison."
 (defun    htype:category (type-category)
   "Return list of symbols in Hyperbole TYPE-CATEGORY in priority order.
 Symbols contain category component.
-TYPE-CATEGORY should be 'actypes, 'ibtypes or nil for all."
+TYPE-CATEGORY should be `actypes', `ibtypes' or nil for all."
   (let ((def-symbols (symset:get type-category 'symbols))
 	(symtable (symtable:select type-category)))
     ;; Expand def-symbols to Elisp symbols by adding prefix
@@ -239,9 +272,9 @@ Return the Hyperbole symbol for the TYPE if it existed, else nil."
   (documentation type))
 
 (defun    htype:names (type-category &optional sym)
-  "Return a list of the current definition names for TYPE-CATEGORY in priority order.
+  "Return a list of current definition names for TYPE-CATEGORY in priority order.
 Definition names do not contain the category prefix.
-TYPE-CATEGORY should be 'actypes, 'ibtypes or nil for all.
+TYPE-CATEGORY should be `actypes', `ibtypes' or nil for all.
 When optional SYM is given, returns the name for that symbol only, if any."
   (let ((types (symset:get type-category 'symbols))
 	(sym-name (when sym (symbol-name sym))))
@@ -255,9 +288,13 @@ When optional SYM is given, returns the name for that symbol only, if any."
 
 ;;; ------------------------------------------------------------------------
 
+;;;###autoload
 (defun   htype:symbol (type type-category)
-  "Return possibly new Hyperbole type symbol composed from TYPE and TYPE-CATEGORY (both symbols)."
-  (intern (concat (symbol-name type-category) "::" (symbol-name type))))
+  "Return possibly new Hyperbole type symbol composed from TYPE and TYPE-CATEGORY.
+TYPE and TYPE-CATEGORY are both symbols.  TYPE-CATEGORY must be one of
+`actypes' or `ibtypes'; if not, return nil."
+  (when (memq type-category '(actypes ibtypes))
+    (intern (concat (symbol-name type-category) "::" (symbol-name type)))))
 
 ;;; ========================================================================
 ;;; action class
@@ -276,7 +313,7 @@ When optional SYM is given, returns the name for that symbol only, if any."
 	       (t function))))
     (cond ((and action (fboundp 'interactive-form))
 	   (interactive-form action))
-	  ((hypb:emacs-byte-code-p action)
+	  ((byte-code-function-p action)
 	   (cond ((fboundp 'compiled-function-interactive)
 		  (compiled-function-interactive action))
 		 ((commandp action)
@@ -293,58 +330,20 @@ When optional SYM is given, returns the name for that symbol only, if any."
   "Return Hyperbole action that execute a keyboard MACRO REPEAT-COUNT times."
   (list 'execute-kbd-macro macro repeat-count))
 
-;; This function is based on Emacs `help-function-arglist'.
 (defun action:params-emacs (def)
-  "Return the argument list for the function DEF which may be a symbol or a function body."
-  ;; Handle symbols aliased to other symbols.
-  (if (and (symbolp def) (fboundp def)) (setq def (indirect-function def)))
-  ;; If definition is a macro, find the function inside it.
-  (if (eq (car-safe def) 'macro) (setq def (cdr def)))
-  (cond
-   ((and (byte-code-function-p def) (listp (aref def 0))) (aref def 0))
-   ((eq (car-safe def) 'lambda) (nth 1 def))
-   ((eq (car-safe def) 'closure) (nth 2 def))
-   ((or (and (byte-code-function-p def) (integerp (aref def 0)))
-	(subrp def))
-    (or (let* ((doc (condition-case nil (documentation def) (error nil)))
-	       (docargs (if doc (car (help-split-fundoc doc nil))))
-	       (arglist (if docargs
-			    (cdar (read-from-string (downcase docargs)))))
-	       (valid t))
-	  ;; Check validity.
-	  (dolist (arg arglist)
-	    (unless (and (symbolp arg)
-			 (let ((name (symbol-name arg)))
-			   (if (eq (aref name 0) ?&)
-			       (memq arg '(&rest &optional))
-			     (not (string-match "\\." name)))))
-	      (setq valid nil)))
-	  (when valid arglist))
-	(let* ((args-desc (if (not (subrp def))
-			      (aref def 0)
-			    (let ((a (subr-arity def)))
-			      (logior (car a)
-				      (if (numberp (cdr a))
-					  (lsh (cdr a) 8)
-					(lsh 1 7))))))
-	       (max (lsh args-desc -8))
-	       (min (logand args-desc 127))
-	       (rest (logand args-desc 128))
-	       (arglist ()))
-	  (dotimes (i min)
-	    (push (intern (concat "arg" (number-to-string (1+ i)))) arglist))
-	  (when (> max min)
-	    (push '&optional arglist)
-	    (dotimes (i (- max min))
-	      (push (intern (concat "arg" (number-to-string (+ 1 i min))))
-		    arglist)))
-	  (unless (zerop rest) (push '&rest arglist) (push 'rest arglist))
-	  (nreverse arglist))))
-   ((and (autoloadp def) (not (eq (nth 4 def) 'keymap)))
-    ;; Force autoload to get function signature.
-    (setq def (autoload-do-load def))
-    (unless (autoloadp def)
-      (action:params-emacs def)))))
+  "Return the argument list for the function DEF.
+DEF may be a symbol or a function body."
+  (let ((params (help-function-arglist def t)))
+    (cond ((listp params) ;; includes nil
+	   params)
+	  ((stringp params)
+	   (when (and (autoloadp def) (not (eq (nth 4 def) 'keymap)))
+	     ;; Force autoload to get function signature.
+	     (setq def (autoload-do-load def))
+	     (unless (autoloadp def)
+	       (action:params-emacs def))))
+	  (t
+	   (error "(action:params-emacs): Construct not supported: %s" def)))))
 
 (defun action:params (action)
   "Return unmodified ACTION parameter list.
@@ -352,11 +351,15 @@ Autoloads action function if need be to get the parameter list."
   (when (and (symbolp action) (fboundp action))
     (setq action (hypb:indirect-function action)))
   (cond ((null action) nil)
+	((fboundp 'help-function-arglist)
+	 (help-function-arglist action t))
 	((listp action)
-	 (if (eq (car action) 'autoload)
-	     (error "(action:params): Autoload not supported: %s" action)
-	   (car (cdr action))))
-	((hypb:emacs-byte-code-p action)
+	 (cond ((eq (car action) 'closure)
+		(nth 2 action))
+	       ((eq (car action) 'autoload)
+		(error "(action:params): Autoload not supported: %s" action))
+	       (t (car (cdr action)))))
+	((byte-code-function-p action)
 	 (if (fboundp 'compiled-function-arglist)
 	     (compiled-function-arglist action)
 	   (action:params-emacs action)))
@@ -370,35 +373,19 @@ Autoloads action function if need be to get the parameter list."
 			  nil param))
 	      (action:params action))))
 
-(defun action:path-args-abs (args-list &optional default-dirs)
-  "Return any paths in ARGS-LIST made absolute.
-Uses optional DEFAULT-DIRS or `default-directory'.
-Other arguments are returned unchanged."
-  (mapcar (lambda (arg) (hpath:absolute-to arg default-dirs))
-	  args-list))
-
-(defun action:path-args-rel (args-list)
-  "Return any paths in ARGS-LIST below button source loc directory made relative.
-Other paths are simply expanded.  Non-path arguments are returned unchanged."
-  (let ((loc (hattr:get 'hbut:current 'loc)))
-    (mapcar (lambda (arg)
-	      (hpath:relative-to arg (if (stringp loc)
-					 (file-name-directory loc)
-				       (buffer-local-value 'default-directory loc))))
-	    args-list)))
-
-
 ;;; ========================================================================
 ;;; action type class, actype
 ;;; ========================================================================
 
-(defmacro hact (&rest args)
+(defun hact (&rest args)
   "Perform action formed from rest of ARGS and return the result.
 The value of `hrule:action' determines what effect this has.
 Alternatively act as a no-op when testing implicit button type contexts.
 First arg may be a symbol or symbol name for either an action type or a
 function.  Runs `action-act-hook' before performing action."
-  (eval `(cons 'funcall (cons 'hrule:action ',args))))
+  ;; (message "hact args: %S" args)
+  (hattr:set 'hbut:current 'actype (actype:elisp-symbol (car args)))
+  (apply hrule:action args))
 
 (defun    actype:act (actype &rest args)
   "Perform action formed from ACTYPE and rest of ARGS and return value.
@@ -406,21 +393,25 @@ If value is nil, however, t is returned instead, to ensure that implicit button
 types register the performance of the action.  ACTYPE may be a symbol or symbol
 name for either an action type or a function.  Runs `action-act-hook' before
 performing ACTION."
+  (when (null actype)
+    (error "(actype:act): No action type specified"))
   (let ((prefix-arg current-prefix-arg)
 	(action (actype:action actype)))
     (if (null action)
-	(error "(actype:act): Null action for: `%s'" actype)
+	(error "(actype:act): Null action for action type: `%s'" actype)
       ;; Next 2 lines are needed so that relative paths are expanded
       ;; properly.  But in rare cases, this can improperly expand simple
       ;; string arguments like "tags" as a pathname, when it is not
       ;; being used as a path.  So do this only if actype is a defact
       ;; and not a defun to limit any potential impact. RSW - 9/22/2017
-      (and (symbolp action) (symtable:actype-p action)
-	   (setq args (action:path-args-abs args)))
+      (and (symbolp action)
+	   (symtable:hyperbole-actype-p action)
+	   (setq args (hpath:absolute-arguments actype args)))
       (let ((hist-elt (hhist:element)))
 	(run-hooks 'action-act-hook)
 	(prog1 (or (if (or (symbolp action) (listp action)
-			   (hypb:emacs-byte-code-p action)
+			   (byte-code-function-p action)
+			   (subrp action)
 			   (and (stringp action) (not (integerp action))
 				(setq action (key-binding action))))
 		       (eval (cons action args))
@@ -429,11 +420,11 @@ performing ACTION."
 	  (hhist:add hist-elt))))))
 
 ;; Return the full Elisp symbol for ACTYPE, which may be a string or symbol.
-(defalias   'actype:elisp-symbol 'symtable:actype-p)
+(defalias 'actype:elisp-symbol #'symtable:actype-p)
 
 (defun    actype:def-symbol (actype)
   "Return the abbreviated symbol for ACTYPE used in its `defact'.
-ACTYPE must be a symbol or string that begins with 'actype::' or nil
+ACTYPE must be a symbol or string that begins with `actype::' or nil
 is returned."
   (let ((name (if (stringp actype)
 		  actype
@@ -443,37 +434,43 @@ is returned."
 
 (defun    actype:eval (actype &rest args)
   "Perform action formed from ACTYPE and rest of ARGS and return value.
+This differs from `actype:act' in that it can return nil and does not
+expand relative pathname ARGS.
+
 ACTYPE may be a string containing a Lisp expression from which ACTYPE
 and ARGS are extracted.  ACTYPE may be a symbol or symbol name for
 either an action type or a function.  Run `action-act-hook' before
 performing ACTION."
   (let ((prefix-arg current-prefix-arg)
-	(action (actype:action actype))
-	(act '(apply action args)))
+	(action (actype:action actype)))
     (if (null action)
 	(error "(actype:act): Null action for: `%s'" actype)
       (let ((hist-elt (hhist:element)))
 	(run-hooks 'action-act-hook)
 	(prog1 (if (or (symbolp action) (listp action)
-		       (hypb:emacs-byte-code-p action)
-		       (and (stringp action) (not (integerp action))
-			    (setq action (key-binding action))))
-		   (eval act)
+			  (byte-code-function-p action)
+			  (subrp action)
+			  (and (stringp action) (not (integerp action))
+			       (setq action (key-binding action))))
+		   (apply action args)
 		 (eval action))
 	  (hhist:add hist-elt))))))
 
 (defun    actype:action (actype)
-  "Return action part of ACTYPE (a bound function symbol, symbol name or function body).
-ACTYPE may be a Hyperbole actype or Emacs Lisp function."
+  "If ACTYPE is a bound function symbol, return it.
+Otherwise, return its body.  ACTYPE must be a bound function
+symbol, symbol name or function body.  ACTYPE may be a Hyperbole
+actype or Emacs Lisp function."
   (let (actname
 	action)
-    (if (stringp actype)
-	(setq actname actype
-	      actype (intern actype))
-      (setq actname (symbol-name actype)))
+    (cond ((stringp actype)
+	   (setq actname actype
+		 actype (intern actype)))
+	  ((and actype (symbolp actype))
+	   (setq actname (symbol-name actype))))
     (setq actype (or (symtable:actype-p actname) actype)
 	  action (htype:body actype))
-    (if (fboundp actype)
+    (if (functionp actype)
 	actype
       action)))
 
@@ -489,41 +486,62 @@ If no action body and actype is a bound function symbol, return that."
     (cond ((htype:body (or (symtable:actype-p actname) actype)))
 	  ((fboundp actype) actype))))
 
-(defmacro actype:create (type params doc &rest default-action)
-  "Create an action TYPE (an unquoted symbol) with PARAMS, described by DOC.
+(defmacro defact (type params doc &rest default-action)
+  "Create Hyperbole action TYPE (unquoted symbol) with PARAMS, described by DOC.
 The type uses PARAMS to perform DEFAULT-ACTION (list of the rest of the
 arguments).  A call to this function is syntactically the same as for
-`defun',  but a doc string is required.
+`defun', but a doc string is required.
 Return symbol created when successful, else nil."
+  (declare (doc-string 3))
   `(progn
      (symtable:add ',type symtable:actypes)
      (htype:create ,type actypes ,doc ,params ,default-action nil)))
 
-(defalias 'defact 'actype:create)
-(put      'actype:create 'lisp-indent-function 'defun)
+(put      'defact 'lisp-indent-function 'defun)
+
+;; Support edebug-defun for interactive debugging of actypes
+(def-edebug-spec defact
+  (&define name lambda-list
+           [&optional stringp]   ; Match the doc string, if present.
+           def-body))
+
+(def-edebug-spec lambda-list
+  (([&rest arg]
+    [&optional ["&optional" arg &rest arg]]
+    &optional ["&rest" arg])))
+
+(defalias 'actype:create #'defact)
 
 (defun    actype:delete (type)
   "Delete an action TYPE (a symbol).  Return TYPE's symbol if it existed."
   (symtable:delete type symtable:actypes)
   (htype:delete type 'actypes))
 
-(defun    actype:doc (hbut &optional full)
-  "Return first line of act doc for HBUT (a Hyperbole button symbol).
+(defun    actype:doc (but &optional full)
+  "Return first line of action doc for BUT.
+BUT should be a Hyperbole button symbol or an Emacs push-button.
 With optional FULL, returns full documentation string.
 Return nil when no documentation."
-  (let* ((act (and (hbut:is-p hbut) (or (hattr:get hbut 'action)
-					(hattr:get hbut 'actype))))
-	 (but-type (hattr:get hbut 'categ))
-	 (sym-p (and act (symbolp act)))
+  (let* ((is-hbut (hbut:is-p but))
+	 (act (if is-hbut
+		  (and (hbut:is-p but) (or (hattr:get but 'action)
+					   (hattr:get but 'actype)))
+		(let ((attrs (hattr:list but)))
+		  (or (plist-get attrs 'action)
+		      (when (plist-get attrs 'follow-link)
+			(plist-get attrs 'help-echo))))))
+	 (but-type (if is-hbut
+		       (hattr:get but 'categ)
+		     act))
+	 (sym-p (when act (symbolp act)))
 	 (end-line) (doc))
-    (cond ((and but-type (fboundp but-type)
-		(setq doc (htype:doc but-type)))
-	   ;; Is an implicit button, so use its doc string if any.
-	   )
+    (cond ((stringp but-type)
+	   (setq doc but-type))
+	  ((and (functionp but-type)
+		(setq doc (htype:doc but-type)))) ;; Is an implicit button, use its doc string.
 	  (sym-p
 	   (setq doc (htype:doc act))))
-    (if (null doc)
-	nil
+    (when doc
       (setq doc (substitute-command-keys doc))
       (or full (setq end-line (string-match "[\n]" doc)
 		     doc (substring doc 0 end-line))))
