@@ -1,10 +1,13 @@
-;;; hactypes.el --- Default action types for GNU Hyperbole
+;;; hactypes.el --- Default action types for GNU Hyperbole  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    23-Sep-91 at 20:34:36
+;; Last-Mod:      3-Dec-23 at 11:37:36 by Bob Weiner
 ;;
-;; Copyright (C) 1991-2016  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -15,8 +18,19 @@
 ;;; Other required Elisp libraries
 ;;; ************************************************************************
 
-(eval-and-compile (mapc #'require '(hbut hpath hargs hmail)))
-(eval-when-compile (mapc #'require '(comint hsettings)))
+(eval-and-compile (mapc #'require '(bookmark hvar hsettings comint hbut hpath hargs hmail man hsys-org)))
+
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function kotl-mode:goto-cell "kotl-mode")
+(declare-function kotl-mode:beginning-of-buffer "kotl-mode")
+(declare-function kotl-mode:goto-cell-ref "kotl-mode")
+(declare-function kcell-view:indent "kcell-view")
+
+(declare-function rmail:msg-to-p "hrmail")
+(declare-function org-roam-id-find "‎ext:org-roam-id")
 
 ;;; ************************************************************************
 ;;; Standard Hyperbole action types
@@ -25,12 +39,17 @@
 (defact annot-bib (key)
   "Follow internal ref KEY within an annotated bibliography, delimiters=[]."
   (interactive "sReference key (no []): ")
-  (let ((opoint (point))
-	(key-regexp (concat "^[*]*[ \t]*\\\[" (ebut:key-to-label key) "\\\]"))
-	citation)
-    (if (save-excursion
-	  (goto-char (point-max))
-	  (setq citation (re-search-backward key-regexp nil t)))
+  (let* ((key-regexp (concat "^[*]*[ \t]*\\[" (ebut:key-to-label key) "\\]"))
+	 (lbl-start (hattr:get 'hbut:current 'lbl-start))
+	 (lbl-end (hattr:get 'hbut:current 'lbl-end))
+	 (citation (when (and lbl-start lbl-end)
+		     (save-excursion
+		       (goto-char (point-max))
+	               (and (re-search-backward key-regexp nil t)
+			    (or (< (point) (1- lbl-start))
+				(> (point) (1+ lbl-end)))
+			    (point))))))
+    (if citation
 	(progn (hpath:display-buffer (current-buffer))
 	       (goto-char citation)
 	       (beginning-of-line))
@@ -41,7 +60,7 @@
 Unless point is at the end of the buffer or if a completion has already been
 inserted, delete the completions window."
   (interactive)
-  (if (eobp)
+  (if (smart-eobp)
       (progn (bury-buffer nil)
 	     (delete-window))
     (hargs:completion)))
@@ -49,20 +68,31 @@ inserted, delete the completions window."
 (defact display-boolean (bool-expr)
   "Display a message showing the result value of a BOOL-EXPR.
 Return any non-nil value or t."
-  (let ((result (eval bool-expr)))
-    (message "Boolean result (%s) = %s" (if result "True" "False") (prin1-to-string result))
+  (interactive "xDisplay bool expr value: ")
+  (let ((result (eval bool-expr t)))
+    (message "Result = %S; Boolean value = %s; Expr = %S"
+	     result (if result "True" "False") bool-expr)
+    (or result t)))
+
+(defact display-value (value)
+  "Display a message showing VALUE (a symbol) and its value.
+Return any non-nil value or t."
+  (interactive "SDisplay symbol's value: ")
+  (let ((result (eval value)))
+    (message "%S" result)
     (or result t)))
 
 (defact display-variable (var)
-  "Display a message showing `var` (a symbol) and its value.
+  "Display a message showing VAR (a symbol) and its value.
 Return any non-nil value or t."
-  (message "%s = %s" var (symbol-value var))
+  (interactive "vDisplay variable's value: ")
+  (message "%s = %S" var (symbol-value var))
   (or (symbol-value var) t))
 
 (defact eval-elisp (lisp-expr)
-  "Evaluate a Lisp expression LISP-EXPR for its side-effects and return any non-nil value."
+  "Evaluate a LISP-EXPR for its side-effects and return any non-nil value."
   (interactive "xLisp to eval: ")
-  (eval lisp-expr))
+  (eval lisp-expr t))
 
 (defact exec-kbd-macro (kbd-macro &optional repeat-count)
   "Execute KBD-MACRO REPEAT-COUNT times.
@@ -102,8 +132,7 @@ error."
 				    (and (integerp repeat) (>= repeat 0))))
 			      1))
      (list macro repeat)))
-  (if (called-interactively-p 'interactive)
-      nil
+  (unless (called-interactively-p 'interactive)
     (or (and kbd-macro (or (stringp kbd-macro)
 			   (and (symbolp kbd-macro) (fboundp kbd-macro))))
 	(hypb:error "(exec-kbd-macro): Bad macro: %s" kbd-macro))
@@ -118,9 +147,9 @@ Optional non-nil second argument INTERNAL-CMD inhibits display of the shell
 command line executed.  Optional non-nil third argument KILL-PREV means
 kill the last output to the shell buffer before executing SHELL-CMD."
   (interactive
-   (let ((default  (car defaults))
-	 (default1 (nth 1 defaults))
-	 (default2 (nth 2 defaults)))
+   (let ((default  (nth 0 hargs:defaults))
+	 (default1 (nth 1 hargs:defaults))
+	 (default2 (nth 2 hargs:defaults)))
      (list (hargs:read "Shell cmd: "
 		       (lambda (cmd) (not (string-equal cmd "")))
 		       default "Enter a shell command." 'string)
@@ -130,18 +159,19 @@ kill the last output to the shell buffer before executing SHELL-CMD."
 			     default2)))))
   (require 'comint)
   (let ((buf-name "*Hyperbole Shell*")
-	(obuf (current-buffer)))
+	(obuf (current-buffer))
+	(default-dir (expand-file-name default-directory)))
     (unwind-protect
 	(progn
-	  (if (not (hpath:remote-p default-directory))
-	      (setq shell-cmd
-		    (concat "cd " default-directory "; " shell-cmd)))
+	  (unless (hpath:remote-p default-dir)
+	    (setq shell-cmd
+		  (concat "cd " default-dir " && " shell-cmd)))
 	  (if (and (get-buffer buf-name)
 		   (get-buffer-process (get-buffer buf-name)))
 	      (hpath:display-buffer buf-name)
 	    ;; (hpath:display-buffer (current-buffer))
-	    (if (eq (minibuffer-window) (selected-window))
-		(other-window 1))
+	    (when (eq (minibuffer-window) (selected-window))
+	      (other-window 1))
 	    ;; 'shell' calls pop-to-buffer which normally displays in
 	    ;; another window
 	    (setq buf-name (buffer-name (shell buf-name))))
@@ -164,17 +194,17 @@ kill the last output to the shell buffer before executing SHELL-CMD."
 (defact exec-window-cmd (shell-cmd)
   "Asynchronously execute an external window-based SHELL-CMD string."
   (interactive
-   (let ((default  (car defaults)))
+   (let ((default  (car hargs:defaults)))
      (list (hargs:read "Shell cmd: "
 		       (lambda (cmd) (not (string-equal cmd "")))
 		       default "Enter a shell command." 'string))))
   (require 'comint)
-  (let ((buf-name "*Hyperbole Shell*")
-	(cmd (if (hpath:remote-p default-directory)
-		 (concat "(" shell-cmd ") &")
-	       (concat "(cd " default-directory "; " shell-cmd ") &")))
-	(msg (format "Executing: %s" shell-cmd))
-	(shell-buf))
+  (let* ((buf-name "*Hyperbole Shell*")
+	 (default-dir (expand-file-name default-directory))
+	 (cmd (if (hpath:remote-p default-dir)
+		  (concat "(" shell-cmd ") &")
+		(concat "(cd " default-dir " && " shell-cmd ") &")))
+	 (msg (format "Executing: %s" shell-cmd)))
     (message msg)
     (save-excursion
       (save-window-excursion
@@ -196,26 +226,13 @@ kill the last output to the shell buffer before executing SHELL-CMD."
 	(comint-send-input)))
     (message msg)))
 
-(defact function-in-buffer (name pos)
-  "Display the definition of function NAME found at POS in the current buffer."
-  (save-excursion
-    (goto-char pos)
-    (unless (looking-at (regexp-quote name))
-      (let ((fume-scanning-message nil))
-	(fume-rescan-buffer)
-	(setq pos (cdr-safe (assoc name fume-funclist))))))
-  (when pos
-    (hpath:display-buffer (current-buffer))
-    (goto-char pos)
-    ;; Move to beginning of the line for compatibility with find-tag.
-    (beginning-of-line)))
-
 (defact hyp-config (&optional out-buf)
-  "Insert Hyperbole configuration information at the end of the current buffer or within optional OUT-BUF."
+  "Insert Hyperbole configuration information at the end of the current buffer.
+Insert in optional OUT-BUF if specified."
   (hypb:configuration out-buf))
 
 (defact hyp-request (&optional out-buf)
-  "Insert into optional OUT-BUF a description of how to subscribe or unsubscribe from a Hyperbole mail list via email."
+  "Insert into optional OUT-BUF how to (un)subscribe from a Hyperbole mail list."
   (save-excursion
     (and out-buf (set-buffer out-buf))
     ;; Allows for insertion prior to user's email signature
@@ -281,42 +298,41 @@ This type of link is for use within a single editor session.  Use
   (hpath:find directory))
 
 (defact link-to-ebut (key &optional key-file)
-  "Perform action given by an explicit button, specified by KEY and optional KEY-FILE.
-KEY-FILE defaults to the current buffer's file name."
+  "Perform explicit button action specified by KEY and optional KEY-FILE.
+Interactively, KEY-FILE defaults to the current buffer's file name."
   (interactive
    (let (but-lbl
          but-file)
      (while (cond ((setq but-file
-			 (read-file-name
-			  "File of button to link to: " nil nil t))
-		   (if (string-equal but-file "")
-		       (progn (beep) t)))
-		  ((not (file-readable-p but-file))
-		   (message "(link-to-ebut): You cannot read `%s'."
-			    but-file)
-		   (beep) (sit-for 3))))
+                         (read-file-name
+                          "File of button to link to: " nil nil t))
+                   (if (string-equal but-file "")
+                       (progn (beep) t)))
+                  ((not (file-readable-p but-file))
+                   (message "(link-to-ebut): You cannot read `%s'."
+                            but-file)
+                   (beep) (sit-for 3))))
      (list (progn
-	     (find-file-noselect but-file)
-	     (while (string-equal "" (setq but-lbl
-					   (hargs:read-match
-					    "Button to link to: "
-					    (ebut:alist but-file)
-					    nil nil nil 'ebut)))
-	       (beep))
-	     (ebut:label-to-key but-lbl))
-	   but-file)))
+             (find-file-noselect but-file)
+             (while (string-equal "" (setq but-lbl
+                                           (hargs:read-match
+                                            "Button to link to: "
+                                            (ebut:alist but-file)
+                                            nil nil nil 'ebut)))
+               (beep))
+             (ebut:label-to-key but-lbl))
+           but-file)))
   (let (but
-	normalized-file)
-  (if key-file
-      (unless (called-interactively-p 'interactive)
-	(setq normalized-file (hpath:normalize key-file)))
-    (setq normalized-file buffer-file-name))
+        normalized-file)
+    (if key-file
+        (setq normalized-file (hpath:normalize key-file))
+      (setq normalized-file buffer-file-name))
 
-    (if (setq but (and key-file (ebut:get key nil normalized-file)))
-	(hbut:act but)
+    (if (setq but (when normalized-file (ebut:get key nil normalized-file)))
+        (hbut:act but)
       (hypb:error "(link-to-ebut): No button `%s' in `%s'"
-		  (ebut:key-to-label key)
-		  key-file))))
+                  (ebut:key-to-label key)
+                  key-file))))
 
 (defact link-to-elisp-doc (symbol)
   "Display documentation for SYMBOL."
@@ -326,74 +342,80 @@ KEY-FILE defaults to the current buffer's file name."
 	((not (or (boundp symbol) (fboundp symbol)))
 	 (hypb:error "(link-to-elisp-doc): `%s' not defined" symbol))
 	(t (let ((temp-buffer-show-function 'switch-to-buffer))
-	     (hpath:display-buffer (current-buffer))
+	     (hpath:display-buffer (help-buffer))
 	     (describe-symbol symbol)))))
 
+(defun  hactypes:link-to-file-interactively ()
+  "Get a path to link to and return it as a one item list."
+  (let ((prev-reading-p hargs:reading-type)
+	(existing-buf t)
+	path-buf)
+    (unwind-protect
+	(let* ((default-directory (or (hattr:get 'hbut:current 'dir)
+				      (file-name-directory
+				       (or (hattr:get 'hbut:current 'loc) ""))
+				      default-directory))
+	       (file-path (or (car hargs:defaults) default-directory))
+	       (file-point (cadr hargs:defaults))
+	       (hargs:reading-type 'file)
+	       ;; If reading interactive inputs from a key series
+	       ;; (puts key events into the unread queue), then don't
+	       ;; insert default-directory into the minibuffer
+	       ;; prompt, allowing time to remove any extra pathname
+	       ;; quotes added in the key series.
+	       (insert-default-directory (not unread-command-events))
+	       ;; Remove any double quotes and whitespace at the
+	       ;; start and end of the path that interactive use may
+	       ;; have introduced.
+	       (path (hpath:trim (read-file-name "Path to link to: "
+						 file-path file-path)))
+	       (orig-path path)
+	       path-line-and-col
+	       line-num
+	       column-num
+	       normalized-path)
+	  ;; Handle if :line:column are included in path.
+	  (setq path-line-and-col (hpath:file-line-and-column path))
+	  (when path-line-and-col
+	    (setq path (nth 0 path-line-and-col)
+		  line-num (nth 1 path-line-and-col)
+		  column-num (nth 2 path-line-and-col)))
+	  ;; Ensure any variables and heading suffixes following
+	  ;; [#,] are removed before doing path matching.
+	  (setq normalized-path (or (hpath:is-p path) path))
+	  (when (not (or (file-name-absolute-p path)
+			 (string-match "\\`\\$\{" path)))
+	    (setq path (concat default-directory path)))
+	  (setq existing-buf (get-file-buffer normalized-path)
+		path-buf (or existing-buf
+			     (and (file-readable-p normalized-path)
+				  (prog1 (set-buffer (find-file-noselect normalized-path t))
+				    (when (integerp file-point)
+				      (goto-char (min (point-max) file-point)))))))
+	  (if (and path-buf (not line-num))
+	      (with-current-buffer path-buf
+		(setq hargs:reading-type 'character)
+		(if (y-or-n-p
+		     (format "y = Display at present position (line %d); n = no position? "
+			     (count-lines 1 (point))))
+		    (list path (point))
+		  (list path)))
+	    (if path-buf
+		(delq nil (list path (save-excursion
+				       (goto-char (point-min))
+				       (forward-line (1- line-num))
+				       (when column-num (move-to-column column-num))
+				       (point))))
+	      (list (or path orig-path)))))
+      (setq hargs:reading-type prev-reading-p)
+      (when (and path-buf (not existing-buf))
+	(kill-buffer path-buf)))))
+
 (defact link-to-file (path &optional point)
   "Display a file given by PATH scrolled to optional POINT.
 If POINT is given, display the buffer with POINT at the top of
-the window."
-  (interactive
-   (let ((prev-reading-p hargs:reading-p)
-	 (existing-buf t)
-	 path-buf)
-     (unwind-protect
-	 (let* ((default-directory (or (hattr:get 'hbut:current 'dir) default-directory))
-		(file-path (or (car defaults) default-directory))
-		(file-point (cadr defaults))
-		(hargs:reading-p 'file)
-		;; If reading interactive inputs from a key series
-		;; (puts key events into the unread queue), then don't
-		;; insert default-directory into the minibuffer
-		;; prompt, allowing time to remove any extra pathname
-		;; quotes added in the key series.
-		(insert-default-directory (not unread-command-events))
-		;; Remove any double quotes and whitespace at the
-		;; start and end of the path that interactive use may
-		;; have introduced.
-		(path (hpath:trim (read-file-name "Path to link to: "
-						  file-path file-path)))
-		(orig-path path)
-		path-line-and-col
-		line-num
-		column-num
-		normalized-path)
-	   ;; Handle if :line:column are included in path.
-	   (setq path-line-and-col (hpath:file-line-and-column path))
-	   (when path-line-and-col
-	     (setq path (nth 0 path-line-and-col)
-		   line-num (nth 1 path-line-and-col)
-		   column-num (nth 2 path-line-and-col)))
-	   ;; Ensure any variables and heading suffixes following
-	   ;; [#,] are removed before doing path matching.
-	   (setq normalized-path (hpath:is-p path))
-	   (when (not (or (file-name-absolute-p path)
-			  (string-match "\\`\\$\{" path)))
-	     (setq path (concat default-directory path)))
-	   (setq existing-buf (get-file-buffer normalized-path)
-		 path-buf (or existing-buf
-			      (and (file-readable-p normalized-path)
-				   (prog1 (set-buffer (find-file-noselect normalized-path t))
-				     (when (integerp file-point)
-				       (goto-char (min (point-max) file-point)))))))
-	   (if (and path-buf (not line-num))
-	       (with-current-buffer path-buf
-		 (setq hargs:reading-p 'character)
-		 (if (y-or-n-p
-		      (format "y = Display at present position (line %d); n = no position? "
-			      (count-lines 1 (point))))
-		     (list path (point))
-		   (list path)))
-	     (if path-buf
-		 (delq nil (list path (save-excursion
-					(goto-char (point-min))
-					(forward-line (1- line-num))
-					(when column-num (move-to-column column-num))
-					(point))))
-	       (list (or path orig-path)))))
-       (setq hargs:reading-p prev-reading-p)
-       (when (and path-buf (not existing-buf))
-	 (kill-buffer path-buf)))))
+the window or as close as possible."
+  (interactive (hactypes:link-to-file-interactively))
   (if path
       (progn
 	;; Remove any double quotes and whitespace at the start and end of
@@ -419,8 +441,8 @@ the window."
 	  (compile-goto-error)
 	(hpath:find-line path line-num))))
 
-(defact link-to-file-line-and-column (path line-num column-num)
-  "Display a file given by PATH scrolled to LINE-NUM with point at COLUMN-NUM."
+(defact link-to-file-line-and-column (path line-num col-num)
+  "Display a file given by PATH scrolled to LINE-NUM with point at COL-NUM."
   (interactive "fPath to link to: \nnDisplay at line number: \nnand column number: ")
   ;; Remove any double quotes and whitespace at the start and end of
   ;; the path that interactive use may have introduced.
@@ -430,16 +452,19 @@ the window."
 	  (error t))
     (if (and (stringp path) (not (file-name-absolute-p path))
 	     (compilation-buffer-p (current-buffer)))
-	(compile-goto-error)
-      (hpath:find-line path line-num))
-    (move-to-column column-num)))
+	(progn (compile-goto-error)
+	       (move-to-column
+		(if (string-match "\\.kotl?\\(\\'\\|#\\)" path)
+		    (+ (kcell-view:indent) col-num)
+		  col-num)))
+      (hpath:find-line path line-num col-num))))
 
-(defact link-to-gbut (key &optional key-file)
+(defact link-to-gbut (key &optional _key-file)
   "Perform an action given by an existing global button, specified by KEY.
 Optional second arg, KEY-FILE, is not used but is for calling
 compatibility with the `hlink' function."
   (interactive
-   (let ((gbut-file (hpath:validate (hpath:substitute-value gbut:file)))
+   (let ((gbut-file (hpath:validate (hpath:substitute-value (gbut:file))))
 	 but-lbl)
      (if (not (file-readable-p gbut-file))
 	 (hypb:error "(link-to-gbut): You cannot read `%s'" gbut-file)
@@ -448,7 +473,7 @@ compatibility with the `hlink' function."
 	       (while (string-equal "" (setq but-lbl
 					     (hargs:read-match
 					      "Global button to link to: "
-					      (mapcar 'list (gbut:label-list))
+					      (mapcar #'list (gbut:label-list))
 					      nil t nil 'gbut)))
 		 (beep))
 	       (hbut:label-to-key but-lbl))))))
@@ -462,59 +487,74 @@ item-name is available.  Filename may be given without the .info
 suffix."
   (interactive "+XInfo (file)index-item-name to link to: ")
   (require 'info)
+  (when (stringp index-item)
+    ;; Remove any tabs or newlines that might be in index-item.
+    (setq index-item (replace-regexp-in-string "[ \t\n\r\f]+" " " index-item t t)))
   (if (and (stringp index-item) (string-match "^(\\([^\)]+\\))\\(.*\\)" index-item))
       (id-info-item index-item)
     (hypb:error "(link-to-Info-index-entry): Invalid Info index item: `%s'" index-item)))
 
 (defact link-to-Info-node (string)
-  "Display an Info node given by STRING or if not found, try to display it as an Info index item.
+  "Display an Info node given by STRING.
+If not found, try to display it as an Info index item.
 STRING must be a string of the form \"(filename)name\".  During
 button creation, completion for both filename and node names is
 available.  Filename may be given without the .info suffix."
   (interactive "+IInfo (file)nodename to link to: ")
   (require 'info)
+  (when (stringp string)
+    ;; Remove any tabs or newlines that might be in string.
+    (setq string (replace-regexp-in-string "[ \t\n\r\f]+" " " string t t)))
   (if (and (stringp string) (string-match "^(\\([^\)]+\\))\\(.*\\)" string))
       (id-info string)
     (hypb:error "(link-to-Info-node): Invalid Info node: `%s'" string)))
 
-(defact link-to-ibut (key &optional key-file point)
-  "Perform an action given by an implicit button, specified by KEY, optional KEY-FILE and POINT.
-KEY-FILE defaults to the current buffer's file and POINT to the current point.
+(defact link-to-ibut (name-key &optional but-src point)
+  "Activate implicit button given by NAME-KEY, optional BUT-SRC and POINT.
+NAME-KEY must be a normalized key for an ibut <[name]>.
+BUT-SRC defaults to the current buffer's file or if there is no
+attached file, then to its buffer name.  POINT defaults to the
+current point.
 
-When creating the button, point must be on the implicit button to which to link
-and its buffer must have a file attached."
+When the button with this action type is created, point must be
+on the implicit button to which to link."
   (interactive
-   (let ((ibut-key (ibut:at-p t)))
-     (if (and ibut-key buffer-file-name)
-	 (list ibut-key buffer-file-name (point))
-       ;; TODO: If default is null below and are creating, rather than modifying,
-       ;; the link, it would be better to throw an error than create
-       ;; an invalid link, but it is difficult to tell which operation
-       ;; is in progress, so ignore this for now.  -- RSW, 01-25-2020
+   (let ((ibut-name-key (ibut:at-p t)))
+     (cond (ibut-name-key
+	    (list ibut-name-key (or buffer-file-name (buffer-name)) (point)))
+	   ;; !! TODO: If default is null below and are creating, rather than editing
+	   ;; the link, it would be better to throw an error than create
+	   ;; an invalid link, but it is difficult to tell which operation
+	   ;; is in progress, so ignore this for now.  -- RSW, 01-25-2020
 
-       ;; When not on an ibut and moddifying the link, use existing arguments
-       (if (and (boundp 'defaults) defaults (listp defaults))
-	   defaults
-	 (list nil nil nil)))))
+	   ;; When not on an ibut and editing the link, use existing arguments
+	   ((and (bound-and-true-p hargs:defaults) (listp hargs:defaults) hargs:defaults)
+	    hargs:defaults)
+	   (t
+	    (hypb:error "(link-to-ibut): Point must be on an implicit button to create a link-to-ibut")))))
+  (when (null name-key)
+    (hypb:error "(link-to-ibut): Point must be on an implicit button to create a link-to-ibut"))
   (let (but
 	normalized-file)
-    (if key-file
-	(unless (called-interactively-p 'interactive)
-	  (setq normalized-file (hpath:normalize key-file)))
-      (setq normalized-file buffer-file-name))
-    (save-excursion
-      (save-restriction
-	(when key-file
-	  (set-buffer (get-file-buffer normalized-file)))
-	(widen)
-	(if (integerp point) (goto-char (min point (point-max))))
-	(setq but (ibut:to key))))
+    (if but-src
+	(unless (and (get-buffer but-src)
+		     (not (buffer-file-name (get-buffer but-src))))
+	  (setq normalized-file (hpath:normalize but-src)))
+      (setq normalized-file (hpath:normalize buffer-file-name)))
+    (when but-src
+      (set-buffer (or (get-buffer but-src) (get-file-buffer normalized-file))))
+    (widen)
+    (when (or (not normalized-file) (hmail:editor-p) (hmail:reader-p))
+      (hmail:msg-narrow))
+    (when (integerp point)
+      (goto-char (min point (point-max))))
+    (setq but (ibut:to name-key))
     (cond (but
 	   (hbut:act but))
-	  (key
-	   (hypb:error "(link-to-ibut): No implicit button `%s' found in `%s'"
-		       (ibut:key-to-label key)
-		       (or key-file (buffer-name))))
+	  (name-key
+	   (hypb:error "(link-to-ibut): No implicit button named `%s' found in `%s'"
+		       (ibut:key-to-label name-key)
+		       (or but-src (buffer-name))))
 	  (t
 	   (hypb:error "(link-to-ibut): Link reference is null/empty")))))
 
@@ -524,25 +564,33 @@ See documentation for `kcell:ref-to-id' for valid cell-ref formats.
 
 If FILE is nil, use the current buffer.
 If CELL-REF is nil, show the first cell in the view."
-  (interactive "fKotl file to link to: \n+KKcell to link to: ")
+  (interactive (hargs:iform-read '(interactive "fKotl file to link to: \n+KKcell to link to: ")))
   (require 'kfile)
-  (cond ((and (stringp cell-ref) (> (length cell-ref) 0)
-	      (eq ?| (aref cell-ref 0)))
-	 ;; Activate view spec in current window.
-	 (kotl-mode:goto-cell cell-ref))
-	((if file
-	     (hpath:find file)
-	   (hpath:display-buffer (current-buffer)))
-	 (if cell-ref
-	     (kotl-mode:goto-cell cell-ref)
-	   (kotl-mode:beginning-of-buffer))
-	 (recenter 0))))
+  ;; May already be in an 'hpath:find' call where the referent buffer
+  ;; has already been made current.  In that case, skip a second
+  ;; display of the buffer.
+  (unless (and (hyperb:stack-frame '(hpath:find))
+	       ;; Perform a loose test that the current buffer
+	       ;; file name matches the path file name since exact
+	       ;; matching of path is likely to be wrong in
+	       ;; certain cases, e.g. with mount point or os path
+	       ;; alterations.
+	       buffer-file-name
+	       file
+	       (or (null file)
+		   (string-empty-p file)
+		   (equal (file-name-nondirectory file)
+			  (file-name-nondirectory buffer-file-name))))
+    (if (stringp file)
+	(hpath:find file)
+      (hpath:display-buffer (current-buffer))))
+  (kotl-mode:goto-cell-ref cell-ref))
 
 (defact link-to-mail (mail-msg-id &optional mail-file)
   "Display mail msg with MAIL-MSG-ID from optional MAIL-FILE.
 See documentation for the variable `hmail:init-function' for
 information on how to specify a mail reader to use."
-  (interactive "+MMail Msg: ")
+  (interactive (hargs:iform-read '(interactive "+MMail Msg: ")))
   (if (not (fboundp 'rmail:msg-to-p))
       (hypb:error "(link-to-mail): Invoke mail reader before trying to follow a mail link")
     (if (and (listp mail-msg-id) (null mail-file))
@@ -562,6 +610,25 @@ information on how to specify a mail reader to use."
 	(hypb:error "(link-to-mail): No msg `%s' in file \"%s\""
 		    mail-msg-id mail-file)))))
 
+(defact link-to-org-id (id)
+  "Display the Org entry, if any, for ID."
+  (when (stringp id)
+    (let* ((inhibit-message t) ;; Inhibit org-id-find status msgs
+	   (m (or (and (featurep 'org-roam) (org-roam-id-find id 'marker))
+		  (org-id-find id 'marker))))
+      (when m
+	(hact 'link-to-org-id-marker m)))))
+
+(defact link-to-org-id-marker (marker)
+  "Display the Org entry, if any, at MARKER.
+See doc of `ibtypes::org-id' for usage."
+    (unless (markerp marker)
+      (error "(link-to-org-id-marker): Argument must be a marker, not %s" marker))
+    (org-mark-ring-push)
+    (hact 'link-to-buffer-tmp (marker-buffer marker) marker)
+    (move-marker marker nil)
+    (org-fold-show-context))
+
 (defact link-to-regexp-match (regexp n source &optional buffer-p)
   "Find REGEXP's Nth occurrence in SOURCE and display location at window top.
 SOURCE is a pathname unless optional BUFFER-P is non-nil, then SOURCE must be
@@ -570,21 +637,21 @@ Return t if found, signal an error if not."
   (interactive "sRegexp to match: \nnOccurrence number: \nfFile to search: ")
   (let ((orig-src source))
     (if buffer-p
-	(if (stringp source)
-	    (setq source (get-buffer source)))
+	(when (stringp source)
+	  (setq source (get-buffer source)))
       ;; Source is a pathname.
       (if (not (stringp source))
 	  (hypb:error
 	   "(link-to-regexp-match): Source parameter is not a filename: `%s'"
 	   orig-src)
-	(setq source (find-file-noselect (hpath:substitute-value source)))))
+	(setq source (hpath:find-noselect source))))
     (if (not (bufferp source))
 	(hypb:error
 	 "(link-to-regexp-match): Invalid source parameter: `%s'" orig-src)
       (hpath:display-buffer source)
       (widen)
       (goto-char (point-min))
-      (if (re-search-forward regexp nil t n)
+      (if (re-search-forward regexp nil t)
 	  (progn (beginning-of-line) (recenter 0) t)
 	(hypb:error
 	 "(link-to-regexp-match): Pattern not found: `%s'" regexp)))))
@@ -593,8 +660,8 @@ Return t if found, signal an error if not."
   "Retrieve and display an Internet rfc given by RFC-NUM.
 RFC-NUM may be a string or an integer."
   (interactive "nRFC number to retrieve: ")
-  (if (or (stringp rfc-num) (integerp rfc-num))
-      (hpath:find (hpath:rfc rfc-num))))
+  (when (or (stringp rfc-num) (integerp rfc-num))
+    (browse-url-emacs (hpath:rfc rfc-num))))
 
 (defact link-to-string-match (string n source &optional buffer-p)
   "Find STRING's Nth occurrence in SOURCE and display location at window top.
@@ -609,9 +676,12 @@ Return t if found, nil if not."
   "Display the Texinfo FILE and NODE (a string).
 FILE may be a string or nil, in which case the current buffer is used."
   (interactive "fTexinfo file to link to: \nsNode within file to link to: ")
+  (when (stringp node)
+    ;; Remove any tabs or newlines that might be in node name.
+    (setq node (replace-regexp-in-string "[ \t\n\r\f]+" " " node t t)))
   (let (node-point)
     (if file
-        (set-buffer (find-file-noselect file))
+        (set-buffer (find-file-noselect (hpath:substitute-value file)))
       (setq file buffer-file-name))
     (save-excursion
       (goto-char (point-min))
@@ -640,52 +710,47 @@ package to display search results."
 Uses `hpath:display-where' setting to control where the man page is displayed."
   (interactive "sManual topic: ")
   (require 'man)
+  (defvar Man-notify-method)
   (let ((Man-notify-method 'meek))
     (hpath:display-buffer (man topic))))
 
-(defact rfc-toc (&optional buf-name opoint)
+(defact rfc-toc (&optional buf-name opoint sections-start)
   "Compute and display summary of an Internet rfc in BUF-NAME.
-Assumes point has already been moved to start of region to summarize.
-Optional OPOINT is point to return to in BUF-NAME after displaying summary."
+Assume point has already been moved to start of region to summarize.
+Optional OPOINT is point to return to in BUF-NAME after displaying
+summary; otherwise, point remains in the toc occurrence buffer.
+Optional SECTIONS-START limits toc entries to those after that point."
   (interactive)
-  (if buf-name
-      (cond ((get-buffer buf-name)
-	     (switch-to-buffer buf-name))
-	    ((let ((buf (get-file-buffer buf-name)))
-	       (when buf
-		 (switch-to-buffer (setq buf-name buf))
-		 t)))
-	    (t (if opoint (goto-char opoint))
-	       (hypb:error "(rfc-toc): Invalid buffer name: %s" buf-name))))
   (let ((sect-regexp "^[ \t]*[1-9][0-9]*\\.[0-9.]*[ \t]+[^ \t\n\r]")
-	(temp-buffer-show-function 'switch-to-buffer)
-	occur-buffer)
-    (hpath:display-buffer (current-buffer))
-    (occur sect-regexp)
-    (setq occur-buffer (set-buffer "*Occur*"))
-    (rename-buffer (format "*%s toc*" buf-name))
+	(rfc-buf-name (buffer-name))
+	(toc-buf-name (format "*toc %s*" buf-name)))
+    (when (get-buffer toc-buf-name)
+      (kill-buffer toc-buf-name))
+    (occur sect-regexp nil (list (cons sections-start (point-max))))
+    (select-window (get-buffer-window "*Occur*"))
+    (rename-buffer toc-buf-name)
     (re-search-forward "^[ ]*[0-9]+:" nil t)
     (beginning-of-line)
-    (delete-region (point-min) (point))
-    (insert "Contents of " (buffer-name occur-buffer) ":\n")
-    (set-buffer-modified-p nil)
-    (set-buffer buf-name)
-    (if opoint (goto-char opoint))))
+    (let ((inhibit-read-only t)
+	  (buffer-read-only))
+      (remove-text-properties (point-min) (point) '(read-only))
+      (delete-region (point-min) (point))
+      (insert "Sections of " rfc-buf-name ":\n")
+      (set-buffer-modified-p nil))
+    (when opoint
+      (switch-to-buffer buf-name)
+      (goto-char opoint))))
 
 (defact text-toc (section)
   "Jump to the text file SECTION referenced by a table of contents entry at point."
   (interactive "sGo to section named: ")
-  (if (stringp section)
-      (progn
-	(actypes::link-to-regexp-match
-	 (concat "^\\*+[ \t]*" (regexp-quote section))
-	 1 (current-buffer) t)
-	(while (and (= (forward-line -1) 0)
-		    (looking-at "[ \t]*[-=][-=]")))
-	(forward-line 1)
-	(recenter 0))))
+  (when (stringp section)
+    (actypes::link-to-regexp-match section 2 (current-buffer) t))
+  (while (and (= (forward-line -1) 0)
+	      (looking-at "[ \t]*[-=][-=]")))
+  (forward-line 1)
+  (recenter 0))
 
 (provide 'hactypes)
 
 ;;; hactypes.el ends here
-
