@@ -1,10 +1,13 @@
-;;; hib-social.el --- Implicit button type for social media/git hashtag and username references
+;;; hib-social.el --- Implicit button type for social media/git hashtag and username references  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    20-Jul-16 at 22:41:34
+;; Last-Mod:     28-Oct-23 at 12:27:26 by Bob Weiner
 ;;
-;; Copyright (C) 2016-2020  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 2016-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -171,57 +174,68 @@
 ;;; ************************************************************************
 
 (eval-when-compile (require 'browse-url))
-(require 'hbut)
 (require 'hargs)
+(require 'hbut)
+(require 'hypb)
+
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function magit-status-setup-buffer "ext:magit")
 
 ;;; ************************************************************************
 ;;; Public variables
 ;;; ************************************************************************
 
+(defgroup hyperbole-buttons nil
+  "Hyperbole explicit, global and implicit button customizations."
+  :group 'hyperbole)
+
 (defcustom hibtypes-social-default-service "twitter"
-  "Lowercase string matching the name of the default social media service to use when none is specified."
+  "Lowercase string matching the social media service to use as a default."
   :type '(radio (const "facebook")
 		(const "git")
 		(const "github")
 		(const "gitlab")
 		(const "instagram")
 		(const "twitter"))
-  :group 'hyperbole-button)
+  :group 'hyperbole-buttons)
 
 (defcustom hibtypes-social-display-function #'browse-url
-  "Function of one argument, a url, to display when a social media reference is activated."
+  "Function of one arg, url, to display when activating a social media reference."
   :type 'function
-  :group 'hyperbole-button)
+  :group 'hyperbole-buttons)
 
 (defcustom hibtypes-git-default-project nil
   "Default project name to associate with any local git commit link."
   :type 'string
-  :group 'hyperbole-button)
+  :group 'hyperbole-buttons)
 
 (defcustom hibtypes-git-use-magit-flag nil
-  "If magit is available, when activating a git directory button, use Magit rather than Dired."
+  "Non-nil means use `magit' rather than `dired' for a git directory button."
   :type 'boolean
-  :group 'hyperbole-button)
+  :group 'hyperbole-buttons)
 
 (defcustom hibtypes-github-default-project nil
   "Default project name to associate with any Github commit link."
   :type 'string
-  :group 'hyperbole-button)
+  :group 'hyperbole-buttons)
 
 (defcustom hibtypes-github-default-user nil
   "Default user name to associate with any Github commit link."
   :type 'string
-  :group 'hyperbole-button)
+  :group 'hyperbole-buttons)
 
 (defcustom hibtypes-gitlab-default-project nil
   "Default project name to associate with any Github commit link."
   :type 'string
-  :group 'hyperbole-button)
+  :group 'hyperbole-buttons)
 
 (defcustom hibtypes-gitlab-default-user nil
   "Default user name to associate with any Github commit link."
   :type 'string
-  :group 'hyperbole-button)
+  :group 'hyperbole-buttons)
 
 ;;; ************************************************************************
 ;;; Private variables
@@ -231,20 +245,18 @@
   '(("\\`\\(fb\\|facebook\\)\\'"  . "https://www.facebook.com/hashtag/%s")
     ("\\`\\(gh\\|github\\)\\'"    . "https://github.com/%s/%s/%s%s")
     ("\\`\\(gl\\|gitlab\\)\\'"    . "https://www.gitlab.com/%s/%s/%s%s")
-    ("\\`\\(gt\\|git\\)\\'"       . "(cd %s; git %s %s)")
+    ("\\`\\(gt\\|git\\)\\'"       . "(cd %s && git %s %s)")
     ("\\`\\(in\\|instagram\\)\\'" . "https://www.instagram.com/explore/tags/%s/")
-    ("\\`\\(tw\\|twitter\\)\\'"   . "https://twitter.com/search?q=%%23%s&src=hashtag")
-)
-  "Alist of (social-media-service-regexp  . expression-to-display-hashtag-reference) elements.")
+    ("\\`\\(tw\\|twitter\\)\\'"   . "https://twitter.com/search?q=%%23%s&src=hashtag"))
+  "Alist of (social-media-service-regexp . to-display-hashtag-reference) elements.")
 
 (defconst hibtypes-social-username-alist
   '(("\\`\\(fb\\|facebook\\)\\'"  . "https://www.facebook.com/%s")
     ("\\`\\(gh\\|github\\)\\'"    . "https://github.com/%s/")
     ("\\`\\(gl\\|gitlab\\)\\'"    . "https://www.gitlab.com/%s/")
     ("\\`\\(in\\|instagram\\)\\'" . "https://www.instagram.com/%s/")
-    ("\\`\\(tw\\|twitter\\)\\'"   . "https://twitter.com/search?q=@%s")
-    )
-  "Alist of (social-media-service-regexp  . url-with-%s-for-username) elements.")
+    ("\\`\\(tw\\|twitter\\)\\'"   . "https://twitter.com/search?q=@%s"))
+  "Alist of (social-media-service-regexp . url-with-%s-for-username) elements.")
 
 ;; Assume at least a 2-character project name
 (defconst hibtypes-git-project-regexp "/?[[:alnum:]]+[-=._/[:alnum:]]*[-=_[:alnum:]]")
@@ -253,7 +265,7 @@
 (defconst hibtypes-social-regexp
   (concat "\\([[:alpha:]]*\\)\\([#@]\\)"
 	  "\\(" hibtypes-git-project-regexp "\\|" hibtypes-git-file-regexp "\\)")
-  "Regular expression that matches a social media/git hashtag or username reference.
+  "Regexp that matches a social media/git hashtag or username reference.
 See `ibtypes::social-reference' for format details.")
 
 (defvar hibtypes-social-inhibit-modes '(texinfo-mode para-mode)
@@ -264,7 +276,7 @@ See `ibtypes::social-reference' for format details.")
 ;;; ************************************************************************
 
 (defib social-reference ()
-  "Display the web page associated with a social hashtag or username reference at point.
+  "Display web page associated with a social hashtag/username reference at point.
 Reference format is:
   [facebook|git|github|gitlab|instagram|twitter]?[#@]<reference> or
   [fb|gt|gh|gl|in|tw]?[#@]<reference>.
@@ -297,9 +309,9 @@ listed in `hibtypes-social-inhibit-modes'."
 					      (mapcar #'car hibtypes-social-hashtag-alist))))))
 		    ;; Heuristic to ensure this is not an email address
 		    (save-match-data
-		      (not (and (looking-at mail-address-regexp)
+		      (not (and (looking-at hypb:mail-address-regexp)
 				(let ((case-fold-search t))
-				  (string-match mail-address-tld-regexp
+				  (string-match hypb:mail-address-tld-regexp
 						(match-string-no-properties 1)))))))))
 
     (save-match-data
@@ -318,8 +330,9 @@ listed in `hibtypes-social-inhibit-modes'."
 
 ;; Don't make this a defact or its arguments may be improperly expanded as pathnames.
 (defun social-reference (service ref-kind-str hashtag-or-username)
-  "Display the web page at social media SERVICE for REF-KIND-STR and HASHTAG-OR-USERNAME.
-REF-KIND-STR is either \"#\" for a hashtag reference or \"@\" for a username reference."
+  "Display the web page at media SERVICE for REF-KIND-STR and HASHTAG-OR-USERNAME.
+REF-KIND-STR is either \"#\" for a hashtag reference or \"@\" for
+a username reference."
   (if (or (null service) (equal service "")) (setq service hibtypes-social-default-service))
   (let ((case-fold-search t)
 	expr-to-format)
@@ -338,7 +351,7 @@ REF-KIND-STR is either \"#\" for a hashtag reference or \"@\" for a username ref
 
 ;; Don't make this a defact or its arguments may be improperly expanded as pathnames.
 (defun github-reference (reference &optional user project)
-  "Display the Github entity associated with REFERENCE and optional USER and PROJECT.
+  "Display Github entity associated with REFERENCE and optional USER and PROJECT.
 REFERENCE is a string of one of the following forms:
     <ref-item>
     <user>/<project>/<ref-item>
@@ -346,19 +359,21 @@ REFERENCE is a string of one of the following forms:
 or  /<project>.
 
 <ref-item> is one of these:
-  one of the words: branches, commits, contributors, issues, people or staff,
-  pulls, status or tags; the associated items are listed;
+  one of the words: branches, commits, contributors, issues,
+  people or staff, pulls, status or tags; the associated items
+  are listed;
 
-  one of the words: branch, commit, issue, pull or tag followed by a '/' or '=' and
-  an item-id; the item is shown;
+  one of the words: branch, commit, issue, pull or tag followed
+  by a '/' or '=' and an item-id; the item is shown;
 
-  an issue reference given by a positive integer, e.g. 92 or prefaced with GH-, e.g. GH-92;
-  the issue is displayed;
+  an issue reference given by a positive integer, e.g. 92 or
+  prefaced with GH-, e.g. GH-92; the issue is displayed;
 
-  a commit reference given by a hex number, 55a1f0; the commit diff is displayed;
+  a commit reference given by a hex number, 55a1f0; the commit
+  diff is displayed;
 
-  a branch or tag reference given by an alphanumeric name, e.g. hyper20; the
-  files in the branch are listed.
+  a branch or tag reference given by an alphanumeric name,
+  e.g. hyper20; the files in the branch are listed.
 
 USER defaults to the value of `hibtypes-github-default-user'.
 If given, PROJECT overrides any project value in REFERENCE.  If no
@@ -445,7 +460,7 @@ PROJECT value is provided, it defaults to the value of
 
 ;; Don't make this a defact or its arguments may be improperly expanded as pathnames.
 (defun gitlab-reference (reference &optional user project)
-  "Display the Gitlab entity associated with REFERENCE and optional USER and PROJECT.
+  "Display Gitlab entity associated with REFERENCE and optional USER and PROJECT.
 REFERENCE is a string of one of the following forms:
     <ref-item>
     <user>/<project>/<ref-item>
@@ -454,21 +469,25 @@ REFERENCE is a string of one of the following forms:
 or  /<project-or-group> (where a group is a collection of projects).
 
 <ref-item> is one of these:
-  one of the words: activity, analytics, boards or kanban, branches, commits, contributors,
-  groups, issues or list, jobs, labels, merge_requests, milestones, pages, pipelines,
-  pipeline_charts, members or people or staff, projects, pulls, schedules, snippets,
-  status or tags; the associated items are listed;
+  one of the words: activity, analytics, boards or kanban,
+  branches, commits, contributors, groups, issues or list, jobs,
+  labels, merge_requests, milestones, pages, pipelines,
+  pipeline_charts, members or people or staff, projects, pulls,
+  schedules, snippets, status or tags; the associated items are
+  listed;
 
-  one of the words: branch, commit(s), issue(s), milestone(s), pull(s), snippet(s) or
-  tag(s) followed by a '/' or '=' and an item-id; the item is shown;
+  one of the words: branch, commit(s), issue(s), milestone(s),
+  pull(s), snippet(s) or tag(s) followed by a '/' or '=' and an
+  item-id; the item is shown;
 
-  an issue reference given by a positive integer, e.g. 92 or prefaced with GL-, e.g. GL-92;
-  the issue is displayed;
+  an issue reference given by a positive integer, e.g. 92 or
+  prefaced with GL-, e.g. GL-92; the issue is displayed;
 
-  a commit reference given by a hex number, 55a1f0; the commit diff is displayed;
+  a commit reference given by a hex number, 55a1f0; the commit
+  diff is displayed;
 
-  a branch or tag reference given by an alphanumeric name, e.g. hyper20; the
-  files in the branch are listed.
+  a branch or tag reference given by an alphanumeric name,
+  e.g. hyper20; the files in the branch are listed.
 
 USER defaults to the value of `hibtypes-gitlab-default-user'.
 If given, PROJECT overrides any project value in REFERENCE.  If no
@@ -607,11 +626,19 @@ PROJECT value is provided, it defaults to the value of
 ;;; Local git repository commit references
 
 (defib git-commit-reference ()
-  "Display the diff for a git commit reference, e.g. \"commit a55e21\", typically produced by git log."
-  (if (save-excursion
-	(beginning-of-line)
-	(looking-at "\\s-*commit \\([0-9a-f]+\\)$"))
-      (hact #'git-reference (match-string-no-properties 1))))
+  "Display the changeset for a commit reference typically produced by git log.
+A git commit reference has the form \"commit a55e21\".
+
+Hyperbole also includes two commands, `hypb:fgrep-git-log' and
+`hypb:grep-git-log' to list git commit references whose changesets
+contain either the string (fgrep) or regular expression (grep) given.
+Then an Action Key displays the associated changeset."
+  (when (save-excursion
+	  (beginning-of-line)
+	  (looking-at "\\(^\\|\\s-+\\)\\(commit #?\\([0-9a-f][0-9a-f][0-9a-f][0-9a-f]+\\)\\)\\(\\s-\\|$\\)"))
+    (save-match-data
+      (ibut:label-set (match-string-no-properties 2) (match-beginning 2) (match-end 2)))
+    (hact #'git-reference (match-string-no-properties 3))))
 
 (defvar hibtypes-git-repos-cache
   (expand-file-name "Local-Git-Repos" hbmap:dir-user)
@@ -626,12 +653,13 @@ PROJECT value is provided, it defaults to the value of
 
 (defun hibtypes-git-build-repos-cache (&optional prompt-flag)
   "Store cache of local git repo directories in `hibtypes-git-repos-cache'.
-With optional PROMPT-FLAG non-nil, prompt user whether to build the cache before building.
-Return t if built, nil otherwise."
+With optional PROMPT-FLAG non-nil, prompt user whether to build
+the cache before building.  Return t if built, nil otherwise."
+  (interactive)
   (when (or (not prompt-flag)
 	    (y-or-n-p "Find all local git repositories (will take some time)? "))
     (message "Please wait while all local git repositories are found...")
-    (unless (zerop (shell-command (format "%s -r '/\.git$' | sed -e 's+/.git$++' > %s"
+    (unless (zerop (shell-command (format "%s -r '/\\.git$' | sed -e 's+/.git$++' > %s"
 					  (hibtypes-git-get-locate-command)
 					  hibtypes-git-repos-cache)))
       (error "(hibtypes-git-build-repos-cache): Cache build failed; `locate-command' must accept `-r' argument for regexp matching"))
@@ -639,8 +667,9 @@ Return t if built, nil otherwise."
     t))
 
 (defun hibtypes-git-add-project-to-repos-cache (project)
-  "Locate PROJECT directory and add to the cache of local git repo directories in `hibtypes-git-repos-cache'.
+  "Locate PROJECT directory and add to directories in `hibtypes-git-repos-cache'.
 Return the project directory found or nil if none."
+  (interactive "sProject: ")
   (message "Please wait while %s's local git repository is found..." project)
   (let ((project-dir (shell-command-to-string
 		      (format "%s -l1 /%s/.git | sed -e 's+/.git++' | tr -d '\n'"
@@ -649,16 +678,17 @@ Return the project directory found or nil if none."
     (message "")
     (when (and (> (length project-dir) 0) (= ?/ (aref project-dir 0)))
       ;; project-dir a directory, prepend it to the cache file...
-      (shell-command-to-string (format "echo -e \"%s\n$(cat %s)\" > %s"
-				       project-dir hibtypes-git-repos-cache
-				       hibtypes-git-repos-cache))
+      (with-current-buffer (find-file-noselect hibtypes-git-repos-cache)
+        (goto-char (point-min))
+        (insert (concat project-dir "\n"))
+        (save-buffer))
       ;; ...and return it.
       project-dir)))
 
-(defun hibtypes-git-build-or-add-to-repos-cache (project &optional prompt-flag)
+(defun hibtypes-git-build-or-add-to-repos-cache (project &optional _prompt-flag)
   "Store cache of local git repo directories in `hibtypes-git-repos-cache'.
-With optional PROMPT-FLAG non-nil, prompt user whether to build the cache before building.
-Return t if built, nil otherwise."
+With optional PROMPT-FLAG non-nil, prompt user whether to build
+the cache before building.  Return t if built, nil otherwise."
   (if (and (file-readable-p hibtypes-git-repos-cache)
 	   ;; Non-zero file size
 	   (not (zerop (nth 7 (file-attributes hibtypes-git-repos-cache)))))
@@ -666,7 +696,7 @@ Return t if built, nil otherwise."
     (hibtypes-git-build-repos-cache t)))
 
 (defun hibtypes-git-project-directory (project)
-  "Given git PROJECT name, return local git repository directory or nil if none found."
+  "Return local git repository directory for PROJECT or nil if none found."
   (if (or (and (file-readable-p hibtypes-git-repos-cache)
 	       ;; Non-zero file size
 	       (not (zerop (nth 7 (file-attributes hibtypes-git-repos-cache)))))
@@ -680,7 +710,7 @@ Return t if built, nil otherwise."
 ;;  1. If within a git repo directory, use that repo unless specified in path
 ;;  2. If project name is given or is default, see if assocated repo dir is in cache and use it.
 ;;  3. Prompt to rebuild locate db and then goto 2 if yes else quit
-;;  4. Run: (cd <dir-found>; git <cmd> <item>)
+;;  4. Run: (cd <dir-found> && git <cmd> <item>)
 ;;  5. Otherwise, do nothing.
 ;;
 ;; Don't make this a defact or its arguments may be improperly expanded as pathnames.
@@ -692,17 +722,20 @@ REFERENCE is a string of one of the following forms:
 or  /<project>.
 
 <ref-item> is one of these:
-  one of the words: branches, commits, or tags; the associated items are listed;
+  one of the words: branches, commits, or tags; the associated
+  items are listed;
 
-  one of the words: branch, commit, or tag followed by a '/' and item id; the item is shown;
+  one of the words: branch, commit, or tag followed by a '/' and
+  item id; the item is shown;
 
-  a commit reference given by a hex number, 55a1f0; the commit diff is displayed;
+  a commit reference given by a hex number, 55a1f0; the commit
+  diff is displayed;
 
-  a branch or tag reference given by an alphanumeric name, e.g. hyper20; the
-  files in the branch are listed.
+  a branch or tag reference given by an alphanumeric name,
+  e.g. hyper20; the files in the branch are listed.
 
-If given, PROJECT overrides any project value in REFERENCE.  If no
-PROJECT value is provided, it defaults to the value of
+If given, PROJECT overrides any project value in REFERENCE.  If
+no PROJECT value is provided, it defaults to the value of
 `hibtypes-git-default-project'."
   (cond ((or (null reference) (equal reference ""))
 	 (error "(git-reference): Git commit hashtag must not be empty"))
@@ -785,6 +818,8 @@ PROJECT value is provided, it defaults to the value of
 		       (setq project-dir (and project (hibtypes-git-project-directory project)))
 		     (error "(git-reference): No git directory found for project `%s'" project)))
 		 (when (equal project-dir "") (setq project-dir nil))
+		 ;; Eliminate ~ that Windows shell can't handle in shell paths
+		 (when project-dir (setq project-dir (expand-file-name project-dir)))
 		 (cond ((and project-dir (file-readable-p project-dir) (file-directory-p project-dir))
 			(if reference
 			    (if (and (equal ref-type "commits") (fboundp 'vc-print-root-log))
@@ -804,7 +839,7 @@ PROJECT value is provided, it defaults to the value of
 							  (if (equal project "") "" " ")
 							  project ref-type
 							  (if (equal reference "") "" " ")
-							  reference)
+							  (substring reference nil (min 9 (length reference))))
 				  (princ (format "Command: %s\n\n" cmd))
 				  (princ (shell-command-to-string cmd)))))
 			  ;; Project-only reference, run dired on the project home directory
@@ -835,9 +870,10 @@ Return nil if no matching file is found."
 (defun hibtypes-git-find (file)
   "Return the shortest pathname matching git-versioned FILE name.
 
-Search for matches in this order: (1) the git repository of the current
-directory, if any; (2) the git repository of project `hibtypes-git-default-project'
-if not nil;  (3) the list of locally cached git repositories in `hibtypes-git-repos-cache'.
+Search for matches in this order: (1) the git repository of the
+current directory, if any; (2) the git repository of project
+`hibtypes-git-default-project' if not nil; (3) the list of
+locally cached git repositories in `hibtypes-git-repos-cache'.
 
 Return nil if no match is found."
   (let (root)
@@ -862,8 +898,9 @@ Return nil if no match is found."
 	nil))))
 
 (defun git-find-file (file)
-  "Locate and edit the named FILE with the shortest git-versioned pathname, typically in another window.
-Uses `hpath:find' to display the FILE.  FILE must not have any path component.
+  "Locate FILE with the shortest git-versioned pathname.
+Uses `hpath:find' to display the FILE, typically in another
+window.  FILE must not have any path component.
 
 If the current directory is in a git repository, search only that one;
 otherwise, search all known local repositories.  Signal an error if no match
