@@ -1,10 +1,13 @@
-;;; hmouse-sh.el --- System-dependent Smart Mouse Key bindings.
+;;; hmouse-sh.el --- System-dependent Smart Mouse Key bindings.  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:     3-Sep-91 at 21:40:58
+;; Last-Mod:      3-Oct-23 at 22:50:40 by Mats Lidell
 ;;
-;; Copyright (C) 1991-2019  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -19,6 +22,32 @@
 ;;; ************************************************************************
 
 (require 'hvar)
+(require 'kmacro)
+
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+(defvar br-env-spec)
+(defvar br-lang-prefix)
+(defvar company-active-map)
+(defvar java-class-def-name-grpn)
+(defvar java-class-def-regexp)
+(defvar jedi-mode)
+
+(defvar hmouse-bindings-flag)           ; "hmouse-key.el"
+(defvar hmouse-previous-bindings)       ; "hmouse-key.el"
+(defvar hmouse-set-point-command)       ; "hui-mouse.el"
+(defvar hmouse-bindings)                ; "hmouse-key.el"
+(defvar hmouse-bindings-flag)           ; "hmouse-key.el"
+
+(defvar Info-mode-map)
+
+(declare-function action-key-depress-emacs "hmouse-drv")
+(declare-function action-mouse-key-emacs "hmouse-drv")
+(declare-function assist-key-depress-emacs "hmouse-drv")
+(declare-function assist-mouse-key-emacs "hmouse-drv")
+(declare-function action-key-depress "hmouse-drv")
+(declare-function hkey-set-key "hyperbole")
 
 ;;; ************************************************************************
 ;;; Public functions
@@ -30,9 +59,18 @@
 ;;   right-divider
 ;;   bottom-divider
 
+;; tab-bar sample Smart Mouse setup
+;; (define-key tab-prefix-map [down-mouse-2] #'action-key-depress-emacs)
+;; (define-key tab-prefix-map [mouse-2]      #'action-mouse-key-emacs)
+
+;; tab-line sample Smart Mouse setup
+;; (define-key tab-line-tab-map [tab-line down-mouse-2] #'action-key-depress-emacs)
+;; (define-key tab-line-tab-map [tab-line mouse-2]      #'action-mouse-key-emacs)
+
 (defun hmouse-bind-key-emacs (mouse-key-number depress-cmd release-cmd)
-  "Ensure MOUSE-KEY-NUMBER (1-5), e.g. 1 for [mouse-1], is globally bound to DEPRESS-CMD and RELEASE-CMD (includes depresses and drags).
-Use nil as cmd values to unbind a key.  Works under GNU Emacs only."
+  "Ensure MOUSE-KEY-NUMBER (1-5) is bound to DEPRESS-CMD and RELEASE-CMD.
+This includes depresses and drags.  Mouse key 1 is [mouse-1], etc.
+Use nil as cmd value to unbind a key."
   (hmouse-set-key-list
    depress-cmd
    (nth (1- mouse-key-number)
@@ -75,8 +113,7 @@ Use nil as cmd values to unbind a key.  Works under GNU Emacs only."
 	   [left-fringe down-mouse-5]
 	   [right-fringe down-mouse-5]
 	   [vertical-line down-mouse-5]
-	   [mode-line down-mouse-5])
-	  )))
+	   [mode-line down-mouse-5]))))
 	   
   (hmouse-set-key-list
    release-cmd
@@ -145,12 +182,12 @@ Use nil as cmd values to unbind a key.  Works under GNU Emacs only."
 	   [vertical-line drag-mouse-5]
 	   [vertical-line mouse-5]
 	   [mode-line drag-mouse-5]
-	   [mode-line mouse-5])
-	  ))))
+	   [mode-line mouse-5])))))
 
 (defun hmouse-bind-shifted-key-emacs (shifted-mouse-key-number depress-cmd release-cmd)
-  "Ensure SHIFTED-MOUSE-KEY-NUMBER (1-5), e.g. 1 for [Smouse-1], is globally bound to DEPRESS-CMD and RELEASE-CMD (includes depresses and drags).
-Use nil as CMD value to unbind the key.  Works under GNU Emacs only."
+  "Ensure SHIFTED-MOUSE-KEY-NUMBER (1-5) is bound to DEPRESS-CMD and RELEASE-CMD.
+This includes depresses and drags.  Shifted Mouse Key 1 is
+[S-mouse-1], etc.  Use nil as cmd value to unbind the key."
   (hmouse-set-key-list
    depress-cmd
    (nth (1- shifted-mouse-key-number)
@@ -183,8 +220,7 @@ Use nil as CMD value to unbind the key.  Works under GNU Emacs only."
 	   [left-fringe S-down-mouse-5]
 	   [right-fringe S-down-mouse-5]
 	   [vertical-line S-down-mouse-5]
-	   [mode-line S-down-mouse-5])
-	  )))
+	   [mode-line S-down-mouse-5]))))
 	  
   (hmouse-set-key-list
    release-cmd
@@ -253,21 +289,21 @@ Use nil as CMD value to unbind the key.  Works under GNU Emacs only."
 	   [vertical-line S-drag-mouse-5]
 	   [vertical-line S-mouse-5]
 	   [mode-line S-drag-mouse-5]
-	   [mode-line S-mouse-5])
-	  ))))
+	   [mode-line S-mouse-5])))))
 	   
-(defun hmouse-get-bindings (hmouse-middle-flag)
-  "Return the list of current bindings of mouse keys used by Hyperbole.
-If HMOUSE-MIDDLE-FLAG is non-nil, includes the middle mouse key binding as well.
-These may be the bindings prior to initializing Hyperbole or the Hyperbole bindings."
+(defun hmouse-get-bindings (middle-flag)
+  "Return the list of active bindings of mouse keys used by Hyperbole.
+If MIDDLE-FLAG is non-nil, include the middle mouse key
+binding as well.  These may be the bindings prior to initializing
+Hyperbole or the Hyperbole bindings."
   ;; Do nothing when running in batch mode.
   (unless noninteractive
     (nconc
-     (when hmouse-middle-flag (hmouse-get-unshifted-bindings))
-     ;; Get mouse bindings under Emacs or XEmacs, even if not under a
-     ;; window system since they can have frames on ttys and windowed
+     (when middle-flag (hmouse-get-unshifted-bindings))
+     ;; Get mouse bindings under Emacs, even if not under a window
+     ;; system, since there can be frames on ttys and windowed
      ;; displays at the same time.
-     (mapcar (lambda (key) (cons key (global-key-binding key)))
+     (mapcar (lambda (key) (cons key (key-binding key)))
 	     (if (eq window-system 'dps)
 		 ;; NEXTSTEP offers only 2 shift-mouse buttons which we use
 		 ;; as the Smart Keys.
@@ -294,8 +330,7 @@ These may be the bindings prior to initializing Hyperbole or the Hyperbole bindi
 		   [mode-line S-down-mouse-1] [mode-line S-drag-mouse-1]
 		   [mode-line S-mouse-1]
 		   [mode-line S-down-mouse-2] [mode-line S-drag-mouse-2]
-		   [mode-line S-mouse-2]
-		   )
+		   [mode-line S-mouse-2])
 	       ;; X, macOS or MS Windows
 	       '([S-down-mouse-2] [S-drag-mouse-2] [S-mouse-2]
 		 [S-down-mouse-3] [S-drag-mouse-3] [S-mouse-3]
@@ -320,11 +355,10 @@ These may be the bindings prior to initializing Hyperbole or the Hyperbole bindi
 		 [mode-line S-down-mouse-2] [mode-line S-drag-mouse-2]
 		 [mode-line S-mouse-2]
 		 [mode-line S-down-mouse-3] [mode-line S-drag-mouse-3]
-		 [mode-line S-mouse-3]
-		 )))
+		 [mode-line S-mouse-3])))
      (nconc
       (mapcar (lambda (key)
-		(cons key (global-key-binding key)))
+		(cons key (key-binding key)))
 	      '([(shift button2)] [(shift button2up)]
 		[(shift button3)] [(shift button3up)]))
       (when (boundp 'mode-line-map)
@@ -335,7 +369,7 @@ These may be the bindings prior to initializing Hyperbole or the Hyperbole bindi
 (defun hmouse-get-unshifted-bindings ()
   "Return the list of middle mouse key bindings prior to their use as Smart Keys."
   ;; Do nothing when running in batch mode.
-  (mapc (lambda (key) (cons key (global-key-binding key)))
+  (mapc (lambda (key) (cons key (key-binding key)))
 	(unless (eq window-system 'dps)
 	  ;; X, macOS or MS Windows
 	  '([down-mouse-2] [drag-mouse-2] [mouse-2]
@@ -359,20 +393,17 @@ These may be the bindings prior to initializing Hyperbole or the Hyperbole bindi
 	    [mode-line down-mouse-2] [mode-line drag-mouse-2]
 	    [mode-line mouse-2]
 	    [mode-line down-mouse-3] [mode-line drag-mouse-3]
-	    [mode-line mouse-3]
-	    )))
+	    [mode-line mouse-3])))
   (nconc
    (mapcar (lambda (key)
-	     (cons key (global-key-binding key)))
+	     (cons key (key-binding key)))
 	   '([button2] [button2up]
-	     [button3] [button3up]
-	     ))
+	     [button3] [button3up]))
    (when (boundp 'mode-line-map)
      (mapcar (function
 	      (lambda (key)
 		(cons key (lookup-key mode-line-map key))))
-	     '([button3] [button3up])))
-   ))
+	     '([button3] [button3up])))))
 
 ;; Based on a function from Emacs mouse.el.
 (defun hmouse-posn-set-point (position)
@@ -385,13 +416,19 @@ Select the corresponding window as well."
     (unless (windowp (posn-window position))
       (error "Position not in text area of window"))
     (select-window (posn-window position)))
-  (when (numberp (posn-point position))
-    (goto-char (posn-point position))))
+  (let ((pos-point (posn-point position)))
+    ;; Need all these checks for vertico-mode
+    (when (and (numberp pos-point)
+	       (>= pos-point (point-min))
+	       (<= pos-point (point-max)))
+      (goto-char pos-point))))
 
 ;; Based on mouse-drag-region from Emacs mouse.el.
 (defun hmouse-drag-region (start-event)
   "Set the region to the text that the mouse is dragged over.
-If not the start of a region drag-and-drop, then depress the Action Key.
+If not the start of a region drag-and-drop, and hyperbole-mode is
+enabled, then depress the Action Key.
+
 Highlight the drag area as you move the mouse.
 This must be bound to a button-down mouse event.
 In Transient Mark mode, the highlighting remains as long as the mark
@@ -407,7 +444,8 @@ is dragged over to."
       (mouse-drag-and-drop-region start-event)
     ;; Give temporary modes such as isearch a chance to turn off.
     (run-hooks 'mouse-leave-buffer-hook)
-    (action-key-depress start-event)
+    (when hyperbole-mode
+      (action-key-depress start-event))
     (mouse-drag-track start-event)))
 
 ;; Based on a function from Emacs mouse.el.
@@ -424,7 +462,8 @@ point determined by `mouse-select-region-move-to-beginning'."
       (with-selected-frame start-w-or-f (setq start-w-or-f (selected-window))))
     (when (framep end-w-or-f)
       (with-selected-frame end-w-or-f (setq end-w-or-f (selected-window))))
-    (if (and (window-minibuffer-p start-w-or-f)
+    (if (and (window-valid-p start-w-or-f)
+	     (window-minibuffer-p start-w-or-f)
 	     (not (minibuffer-window-active-p start-w-or-f)))
 	;; Select the ending frame only, not the window pressed within.
 	(select-frame (window-frame end-w-or-f))
@@ -438,29 +477,31 @@ point determined by `mouse-select-region-move-to-beginning'."
 		     (exchange-point-and-mark))))
 	;; Use event-end in case called from mouse-drag-region.
 	;; If EVENT is a click, event-end and event-start give same value.
-	(if (and (window-minibuffer-p end-w-or-f)
+	(if (and (window-valid-p end-w-or-f)
+		 (window-minibuffer-p end-w-or-f)
 		 (not (minibuffer-window-active-p end-w-or-f)))
 	    ;; Select the ending frame only, not the window pressed within.
 	    (select-frame (window-frame end-w-or-f))
 	  (condition-case ()
 	      (hmouse-posn-set-point (event-end event))
-	    (error (select-frame (window-frame end-w-or-f)))))))))
+	    (error (when (window-valid-p end-w-or-f)
+		     (select-frame (window-frame end-w-or-f))))))))))
 
 (defun hmouse-move-point-eterm (arg-list)
   (apply 'mouse-move-point arg-list))
 
 (defun hmouse-set-key-list (binding key-list)
-  (mapc (lambda (key) (hkey-global-set-key key binding)) key-list)
+  (mapc (lambda (key) (hkey-set-key key binding)) key-list)
   nil)
 
-(defun hmouse-shifted-setup (hmouse-middle-flag)
+(defun hmouse-shifted-setup (middle-flag)
   "Call `hmouse-install' instead of this and see its documentation."
   (interactive)
   ;; Do nothing when running in batch mode.
   (unless noninteractive
     (or hmouse-bindings-flag hmouse-previous-bindings
-	(setq hmouse-previous-bindings (hmouse-get-bindings hmouse-middle-flag)))
-    (when hmouse-middle-flag (hmouse-unshifted-setup hmouse-middle-flag))
+	(setq hmouse-previous-bindings (hmouse-get-bindings middle-flag)))
+    (when middle-flag (hmouse-unshifted-setup middle-flag))
     ;; Ensure Gillespie's Info mouse support is off since
     ;; Hyperbole handles that.
     (when (boundp 'Info-mouse-support) (setq Info-mouse-support nil))
@@ -471,7 +512,7 @@ point determined by `mouse-select-region-move-to-beginning'."
     ;; this.
     (setq kmacro-call-mouse-event nil)
     ;;
-    (setq hmouse-set-point-command 'hmouse-move-point-emacs)
+    (setq hmouse-set-point-command #'hmouse-move-point-emacs)
     (if (eq window-system 'dps)
 	;; NEXTSTEP offers only 2 shift-mouse buttons which we use as the Smart Keys.
 	(progn
@@ -485,11 +526,11 @@ point determined by `mouse-select-region-move-to-beginning'."
 	(define-key company-active-map [S-mouse-2] 'smart-company-to-definition)
 	(define-key company-active-map [S-down-mouse-3] 'ignore)
 	(define-key company-active-map [S-mouse-3] 'smart-company-help)))
-    (setq hmouse-bindings (hmouse-get-bindings hmouse-middle-flag)
+    (setq hmouse-bindings (hmouse-get-bindings middle-flag)
 	  hmouse-bindings-flag t)))
 
 (defun hmouse-unshifted-setup (&optional middle-key-only-flag)
-  "Bind the middle mouse key as the Action Key and the right mouse key as the Assist Key.
+  "Bind the middle and right mouse keys as Action and Assist Keys, respectively.
 With optional MIDDLE-KEY-ONLY-FLAG non-nil, bind only the middle mouse key."
   (interactive)
   ;; Globally Emacs uses key-translation-map to link mouse-1 to
