@@ -1,10 +1,13 @@
-;;; hsettings.el --- Hyperbole settings which may require customization
+;;; hsettings.el --- GNU Hyperbole settings which may require customization  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    15-Apr-91 at 00:48:49
+;; Last-Mod:      3-Oct-23 at 17:22:51 by Mats Lidell
 ;;
-;; Copyright (C) 1991-2019  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1991-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -24,6 +27,20 @@
 ;;; ************************************************************************
 
 (require 'hversion)
+(require 'hvar)
+(require 'browse-url)
+
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(defvar helm-allow-mouse)
+(defvar htz:local)                      ; "htz.el"
+
+(declare-function hproperty:but-create "hui-em-but")
+(declare-function hproperty:but-flash "hui-em-but")
+(declare-function hyperbole-minibuffer-menu "hui-mini")
+(declare-function hyperbole-menubar-menu "hui-menu")
 
 ;;; Read the comments and modify as desired.
 
@@ -78,9 +95,9 @@
 ;; The Smart Menu system is an attractive in-buffer menu system that
 ;; predates Emacs menu systems; it is included in InfoDock.
 (defvar hkey-always-display-menu nil
-  "*Non-nil means always display the Smart Menu window when the Action or Assist Key is pressed and the Smart Menu system has been loaded.
-If a Smart Menu is already displayed, perform another Action or Assist Key
-function.")
+  "*Non-nil means a Smart Key press pops up the Smart Menu window.
+The Smart Menu system must have already been loaded.  If a Smart
+Menu is already displayed, perform another Action or Assist Key function.")
 
 (defcustom hmouse-middle-flag (and (boundp 'infodock-version) infodock-version t)
   "*Under InfoDock or when t, additionally bind the middle mouse button as an
@@ -89,11 +106,12 @@ Action Key."
   :group 'hyperbole-keys)
 
 (defcustom smart-scroll-proportional t
-  "*Non-nil means Smart Keys should scroll relative to current line when pressed at the end of a line.
-Action Key moves current line to top of window.  Assist Key moves current
-line to bottom of window.  Repeated presses then scroll up or down a
-windowful.  Nil value instead ignores current line and always scrolls up or
-down a windowful."
+  "*Non-nil means Smart Key scroll behavior is relative to current line.
+Smart Keys will scroll relative to current line when pressed at
+the end of a line.  Action Key moves current line to top of the
+window.  Assist Key moves current line to bottom of the window.
+Repeated presses then scroll up or down a windowful.  Nil value
+ignores current line and always scrolls up or down a windowful."
   :type 'boolean
   :group 'hyperbole-keys)
 
@@ -120,6 +138,9 @@ down a windowful."
 
 (defun hyperbole-update-menus ()
   "Rebuild all Hyperbole menus with any updated settings."
+  (interactive)
+  (load "hui-menu")
+  (load "hui-mini")
   (hyperbole-menubar-menu)
   (hyperbole-minibuffer-menu))
 
@@ -133,11 +154,14 @@ lines"
 (defun hyperbole-default-web-search-term ()
   "Return a default search term if region is active and not too large."
   (and (region-active-p)
-       (<= (count-lines (region-beginning) (region-end)) hyperbole-default-web-search-term-max-lines)
+       (<= (count-lines (region-beginning) (region-end))
+	   hyperbole-default-web-search-term-max-lines)
        (buffer-substring-no-properties (region-beginning) (region-end))))
 
 (defun hyperbole-read-web-search-arguments (&optional service-name search-term)
-  "Read from the keyboard a list of (web-search-service-string search-term-string) if not given as arguments."
+  "Read from the keyboard a list of (web-search-service-string search-term-string).
+With optional non-empty SERVICE-NAME and SEARCH-TERM arguments,
+use those instead of reading from the keyboard."
   (let ((completion-ignore-case t))
     (while (or (not (stringp service-name)) (equal service-name ""))
       (setq service-name (completing-read "Search service: " hyperbole-web-search-alist
@@ -147,7 +171,7 @@ lines"
 				     (hyperbole-default-web-search-term))))
     (list service-name search-term)))
 
-(defun hyperbole-web-search (&optional service-name search-term)
+(defun hyperbole-web-search (&optional service-name search-term return-search-expr-flag)
   "Search web SERVICE-NAME for SEARCH-TERM.
 Both arguments are optional and are prompted for when not given or when null.
 Uses `hyperbole-web-search-alist' to match each service to its search url.
@@ -156,13 +180,48 @@ package to display search results."
   (interactive)
   (cl-multiple-value-bind (service-name search-term)
       (hyperbole-read-web-search-arguments service-name search-term)
-    (if (assoc service-name hyperbole-web-search-alist)
-	(let ((browse-url-browser-function
-	       hyperbole-web-search-browser-function))
-	  (browse-url
-	   (format (cdr (assoc service-name hyperbole-web-search-alist))
-		   (browse-url-url-encode-chars search-term "[*\"()',=;?% ]"))))
-      (user-error "(Hyperbole): Invalid web search service `%s'" service-name))))
+    (let ((browse-url-browser-function hyperbole-web-search-browser-function)
+	  (search-pat (cdr (assoc service-name hyperbole-web-search-alist
+				  (lambda (service1 service2)
+				    (equal (downcase service1) (downcase service2)))))))
+      (setq search-term (browse-url-url-encode-chars search-term "[*\"()',=;?% ]"))
+      (if return-search-expr-flag
+	  (cond ((stringp search-pat)
+		 (format search-pat search-term))
+		((functionp search-pat)
+		 (list search-pat search-term))
+		(t (user-error "(Hyperbole): Invalid web search service `%s'" service-name)))
+	(cond ((stringp search-pat)
+	       (browse-url (format search-pat search-term)))
+	      ((functionp search-pat)
+	       (funcall search-pat search-term))
+	      (t (user-error "(Hyperbole): Invalid web search service `%s'" service-name)))))))
+
+;; This must be defined before the defcustom `inhbit-hyperbole-messaging'.
+;;;###autoload
+(defun hyperbole-toggle-messaging (&optional arg)
+  "Toggle Hyperbole support for explicit buttons in mail and news buffers.
+Toggle the boolean variable `inhibit-hyperbole-messaging’ and either
+add hooks (nil value) or remove them (t value).
+
+With optional prefix ARG > 0, enable support.  If ARG <= 0,
+disable/inhibit support."
+  (interactive "P")
+  (setq inhibit-hyperbole-messaging (if (null arg)
+					(not inhibit-hyperbole-messaging)
+				      (<= (prefix-numeric-value arg) 0)))
+  (if inhibit-hyperbole-messaging
+      (var:remove-all)
+    (var:append-all)
+    ;; Add any hooks that were skipped when inhibit-hyperbole-messaging
+    ;; was nil.
+    (cond ((boundp 'hyperbole-loading))
+	  ((not after-init-time)
+	   (add-hook 'after-init-hook (lambda () (load "hyperbole"))))
+	  (t (load "hyperbole"))))
+  (if (called-interactively-p 'interactive)
+      (message "Hyperbole messaging button support is %s"
+	       (if inhibit-hyperbole-messaging "disabled" "enabled"))))
 
 (defcustom inhibit-hyperbole-messaging t
   "*Determine whether Hyperbole supports explicit buttons in mail and news buffers.
@@ -177,14 +236,14 @@ to your personal Emacs initialization file, prior to loading
 Hyperbole, and then restart Emacs."
   :type 'boolean
   :initialize #'custom-initialize-set
-  :set (lambda (symbol value)
+  :set (lambda (_symbol value)
 	 ;; Invert value to produce ARG for hyperbole-toggle-messaging.
 	 (hyperbole-toggle-messaging (if value 0 1)))
   :group 'hyperbole-buttons)
 
 (defcustom hyperbole-web-search-browser-function browse-url-browser-function
   "*Function of one url argument called by any Hyperbole Find/Web search."
-  :type 'boolean
+  :type 'function
   :group 'hyperbole-commands)
 
 (defcustom hyperbole-web-search-alist
@@ -197,28 +256,31 @@ Hyperbole, and then restart Emacs."
     ("Facebook" . "https://www.facebook.com/hashtag/%s")
     ;; To search for a Facebook user, use "https://www.facebook.com/%s".
     ("Google" . "http://www.google.com/search?q=%s")
-    ("Hub(git)" . "https://github.com/search?ref=simplesearch&q=%s")
+    ("gitHub" . "https://github.com/search?ref=simplesearch&q=%s")
     ("Images" . "http://www.google.com/images?hl=en&q=%s")
+    ("Jump"   . webjump)
     ("Maps" . "http://maps.google.com/maps?q=%s")
     ("RFCs" . "https://tools.ietf.org/html/rfc%s")
     ("StackOverflow" . "https://stackoverflow.com/search?q=%s")
     ("Twitter" . "https://twitter.com/search?q=%s")
     ("Wikipedia" . "https://en.wikipedia.org/wiki/%s")
     ("Youtube" . "https://www.youtube.com/results?search_query=%s"))
-  "*Alist of (web-service-name . url-with-%s-parameter) elements.
-The first character of each web-service-name must be unique.
+  "*Alist of (web-service-name . emacs-cmd-or-url-with-%s-parameter) elements.
+The first capitalized character of each web-service-name must be unique.
 This custom option is used in the Hyperbole Find/Web menu where
 the %s in the url-with-%s-parameter is replaced with an interactively
-obtained search string."
+obtained search term; if second argument is a command instead, then
+it is called interactively to prompt for the search term with which it
+then runs the search."
   :initialize #'custom-initialize-default
-  :set (lambda (_option value)
-	 (setq hyperbole-web-search-alist value)
+  :set (lambda (option value)
+	 (set option value)
 	 (hyperbole-update-menus))
-  :type '(alist :key-type string :value-type string)
+  :type '(alist :key-type string :value-type (choice string symbol))
   :group 'hyperbole-commands)
 
 ;;; ************************************************************************
-;;; GNU EMACS AND XEMACS CONFIGURATION
+;;; GNU EMACS CONFIGURATION
 ;;; ************************************************************************
 
 ;; No-op unless set by one of the conditionals below.
@@ -228,32 +290,16 @@ obtained search string."
        (require 'hui-em-but)
        ;; Highlight explicit buttons whenever a file is read in.
        (add-hook 'find-file-hook #'hproperty:but-create t)
-       (defalias 'hui:but-flash 'hproperty:but-flash)
-       ;;
-       ;; Substitute for the nil argument below a valid X color name with
-       ;; which to highlight buttons if the default highlighting does not
-       ;; appeal to you. See "hui-em-but.el" for how this works.
-       (hproperty:cycle-but-color nil)
-       ;;
-       ;; Non-nil means highlight all explict buttons with `hproperty:but-face'.
-       (setq hproperty:but-highlight-flag t)
-       ;;
-       ;; Non-nil means visually emphasize that button under mouse cursor is
-       ;; selectable.
-       (setq hproperty:but-emphasize-flag nil)
-       ;;
-       ;; If you find that the Hyperbole button flash time is too slow
-       ;; or too fast, adjust it here.
-       (setq hproperty:but-flash-time 1000)))
+       (defalias 'hui:but-flash #'hproperty:but-flash)))
 
 ;;; ************************************************************************
 ;;; ONLINE LIBRARY CONFIGURATION
 ;;; ************************************************************************
 
-;;; Support for online library document id references is loaded here but
-;;; requires some additional configuration before use.  See the DESCRIPTION
-;;; section in "hib-doc-id.el" for complete installation and use information.
-;;;
+;; Support for online library document id references is loaded here but
+;; requires some additional configuration before use.  See the DESCRIPTION
+;; section in "hib-doc-id.el" for complete installation and use information.
+;;
 (add-hook 'hibtypes-end-load-hook (lambda () (require 'hib-doc-id)))
 
 ;;; ************************************************************************
