@@ -1,10 +1,13 @@
-;;; hui-select.el --- Select larger and larger syntax-driven regions in a buffer.
+;;; hui-select.el --- Select larger and larger syntax-driven regions in a buffer.  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;;
 ;; Orig-Date:    19-Oct-96 at 02:25:27
+;; Last-Mod:     30-Nov-23 at 11:33:09 by Bob Weiner
 ;;
-;; Copyright (C) 1996-2019  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1996-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -74,9 +77,8 @@
 ;;   ---------------
 ;;   SETUP IF USED SEPARATELY FROM HYPERBOLE (otherwise ignore):
 ;;
-;;   To autoload this package under Emacs, XEmacs or InfoDock via
-;;   mouse usage, add the following line to one of your initialization
-;;   files.
+;;   To autoload this package via mouse usage add the following line
+;;   to one of your initialization files.
 ;;
 ;;      (hui-select-initialize)
 ;;
@@ -107,8 +109,9 @@
 
 (require 'hvar)
 (eval-when-compile
-  (require 'sgml-mode) ;; for HTML mode
-  (require 'nxml-mode) ;; for XML mode
+  (require 'mhtml-mode) ;; for MHTML and HTML modes
+  (require 'sgml-mode)  ;; for SGML mode
+  (require 'nxml-mode)  ;; for XML mode
   (require 'web-mode nil t))
 
 ;;; ************************************************************************
@@ -122,7 +125,7 @@
   :group 'hyperbole-commands)
 
 (defcustom hui-select-markup-modes
-  '(html-mode sgml-mode nxml-mode web-mode)
+  '(html-mode mhtml-mode nxml-mode sgml-mode web-mode)
   "*List of markup language modes that use SGML-style <tag> </tag> pairs."
   :type '(repeat (function :tag "Mode"))
   :group 'hyperbole-commands)
@@ -137,13 +140,15 @@
   (append '(altmath-mode asm-mode csh-mode eiffel-mode ksh-mode
                          math-mode miranda-mode python-mode pascal-mode sather-mode)
 	  hui-select-text-modes)
-  "*List of language major modes that use indentation mostly to define syntactic structure."
+  "*List of modes that use indentation mostly to define syntactic structure.
+Use for language major modes."
   :type '(repeat (function :tag "Mode"))
   :group 'hyperbole-commands)
 
 (defcustom hui-select-ignore-quoted-sexp-modes
   '(debugger-mode emacs-lisp-mode lisp-mode lisp-interaction-mode slime-mode cider-mode)
-  "*List of language major modes in which to ignore quoted sexpressions for syntactic matches."
+  "*List of modes in which to ignore quoted sexpressions for syntactic matches.
+Use for language major modes."
   :type '(repeat (function :tag "Mode"))
   :group 'hyperbole-commands)
 
@@ -163,9 +168,9 @@
     (indented-text-mode "[^ \t\n*]")
     (Info-mode "[^ \t\n]")
     (outline-mode "[^*]")
-    (text-mode  "[^ \t\n*]")
-    )
-  "List of (major-mode . non-terminator-line-regexp) elements used to avoid early dropoff when marking indented code.")
+    (text-mode  "[^ \t\n*]"))
+  "List of (MAJOR-MODE . NON-TERMINATOR-LINE-REGEXP) elements.
+Used to avoid early dropoff when marking indented code.")
 
 (defvar hui-select-indent-end-regexp-alist
   '((altmath-mode "[ \t\n]*\n\\S-")
@@ -179,9 +184,9 @@
     (fundamental-mode "[ \t]*$")
     (indented-text-mode "[ \t]*$")
     (Info-mode "[ \t]*$")
-    (text-mode  "[ \t]*$")
-    )
-  "List of (major-mode . terminator-line-regexp) elements used to include a final line when marking indented code.")
+    (text-mode  "[ \t]*$"))
+  "List of (MAJOR-MODE . TERMINATOR-LINE-REGEXP) elements.
+Used to include a final line when marking indented code.")
 
 (defcustom hui-select-char-p nil
   "*If t, return single character boundaries when all else fails."
@@ -198,10 +203,128 @@
   :type 'boolean
   :group 'hyperbole-commands)
 
+(defconst hui-java-defun-prompt-regexp
+  (let* ((space* "[ \t\n\r\f]*")
+         (space+ "[ \t\n\r\f]+")
+         (modifier*
+          (concat "\\(?:"
+                  (regexp-opt '("abstract" "const" "default" "final" "native"
+                                "private" "protected" "public" "static"
+                                "strictfp" "synchronized" "threadsafe"
+                                "transient" "volatile")
+                              'words)   ; Compatible with XEmacs
+                  space+ "\\)*"))
+         (ids-with-dots "[_$a-zA-Z][_$.a-zA-Z0-9]*")
+         (ids-with-dot-\[\] "[[_$a-zA-Z][][_$.a-zA-Z0-9]*")
+         (paren-exp "([^);{}]*)")
+         (generic-exp "<[^(){};]*>"))
+    (concat "^[ \t]*"
+            modifier*
+            "\\(?:" generic-exp space* "\\)?"
+            ids-with-dot-\[\] space+                ; first part of type
+            "\\(?:" ids-with-dot-\[\] space+ "\\)?" ; optional second part of type.
+            "\\(?:[_a-zA-Z][^][ \t:;.,{}()=<>]*"    ; defun name
+                "\\|" ids-with-dots
+            "\\)" space*
+            paren-exp
+            "\\(?:" space* "]\\)*"      ; What's this for?
+            "\\(?:" space* "\\<throws\\>" space* ids-with-dot-\[\]
+                  "\\(?:," space* ids-with-dot-\[\] "\\)*"
+            "\\)?"
+            space*)))
+
 (defvar hui-select-previous nil)
 (defvar hui-select-prior-point nil)
 (defvar hui-select-prior-buffer nil)
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(defvar help-mode-syntax-table)         ; "help-mode.el"
+(defvar hkey-init)                      ; "hyperbole.el"
+(defvar hkey-value)                     ; "hui-mouse.el"
+(defvar hyperbole-mode-map)             ; "hyperbole.el"
+(defvar org-mode-map)                   ; "org.el"
+
+(declare-function kview:valid-position-p "kotl/kview")
+(declare-function hkey-set-key "hyperbole")
+(declare-function hypb:cmd-key-vector "hypb")
+
+;;; ************************************************************************
+;;; Private variables
+;;; ************************************************************************
+
+(defconst hui-select-syntax-table
+  (let ((st (copy-syntax-table emacs-lisp-mode-syntax-table)))
+    ;; Make braces be thing delimiters, not punctuation.
+    (modify-syntax-entry ?\{ "(}" st)
+    (modify-syntax-entry ?\} "){" st)
+    (modify-syntax-entry ?< "(>" st)
+    (modify-syntax-entry ?> ")<" st)
+    st)
+  "Syntax table to use when selecting delimited things.")
+
+(defvar hui-select-bigger-alist
+  '((char			 nil)
+    (whitespace			 hui-select-whitespace)
+    (word			 hui-select-word)
+    (symbol			 hui-select-symbol)
+    (punctuation		 nil)
+    (string			 hui-select-string)
+    (text			 nil)
+    (comment			 hui-select-comment)
+    (markup-pair		 nil)
+    (preprocessor-def		 nil)
+    (sexp			 hui-select-sexp)
+    (sexp-start			 nil)
+    (sexp-end			 nil)
+    (sexp-up			 hui-select-sexp-up)
+    (line			 hui-select-line)
+    (sentence			 hui-select-sentence)
+    (brace-def-or-declaration	 hui-select-brace-def-or-declaration)
+    (indent-def			 hui-select-indent-def)
+    (paragraph			 hui-select-paragraph)
+    (page			 hui-select-page)
+    (buffer			 hui-select-buffer))
+  "Unordered list of (<region-type-symbol> <region-selection-function>) pairs.
+Used to go from one thing to a bigger thing.  See `hui-select-bigger-thing'.
+Nil value for <region-selection-function> means that region type is skipped
+over when trying to grow the region and is only used when a selection is made
+with point on a character that triggers that type of selection.")
+
+(defvar hui-select-prior-buffer nil)
+(defvar hui-select-prior-point nil)
+
+(defvar hui-select-previous 'char
+  "Most recent type of selection.  Must be set by all hui-select functions.")
+
+(defvar hui-select-region (cons nil nil)
+  "Cons cell that contains a region (<beginning> . <end>).
+The function `hui-select-set-region' updates and returns it.")
+
+(defvar hui-select-old-region (cons nil nil)
+  "Cons cell that contains a region (<beginning> . <end>).")
+
+(defcustom hui-select-syntax-alist
+  '((?w  . hui-select-word)
+    (?_  . hui-select-symbol)
+    (?\" . hui-select-string)
+    (?\( . hui-select-sexp-start)
+    (?\$ . hui-select-sexp-start)
+    (?\' . hui-select-sexp-start)
+    (?\) . hui-select-sexp-end)
+    (?   . hui-select-whitespace)
+    (?\< . hui-select-comment)
+    (?\. . hui-select-punctuation))
+  "*Unordered list of pairs of the form (<syntax-char> <function>).
+Used by the function `hui-select-syntactical-region'.
+Each <function> takes a single position argument and returns a
+region (start . end) defining the boundaries of the thing at that position."
+  :type '(repeat (cons (character :tag "Syntax-Char") function))
+  :group 'hyperbole-commands)
+
+
 ;;; ************************************************************************
 ;;; Public functions
 ;;; ************************************************************************
@@ -212,10 +335,11 @@
 
 ;;;###autoload
 (defun hui-select-at-p (&optional pos)
-  "Return non-nil if the character after optional POS (or point) matches a syntax entry in `hui-select-syntax-alist'.
-The non-nil value returned is the function to call to select that syntactic unit."
+  "Non-nil means character matches a syntax entry in `hui-select-syntax-alist'.
+The character is after optional POS or point.  The non-nil value
+returned is the function to call to select that syntactic unit."
   (interactive "d")
-  (unless (and (bobp) (eobp))
+  (unless (smart-eobp)
     (or (numberp pos) (setq pos (point)))
     (setq hui-select-previous 'char)
     (let* ((syntax (char-syntax (or (char-after pos) (char-before pos))))
@@ -229,16 +353,33 @@ The non-nil value returned is the function to call to select that syntactic unit
 
 ;;;###autoload
 (defun hui-select-goto-matching-delimiter ()
-  "Jump back and forth between the start and end delimiters of a thing."
+  "Jump back and forth between the start and end delimiters of a thing.
+
+If the key that invokes this command in `hyperbole-minor-mode' is
+also bound in the current major mode map, then interactively
+invoke that command instead.  Typically prevents clashes over
+{C-c .}."
   (interactive)
-  (cond ((memq major-mode hui-select-markup-modes)
-	 (hui-select-goto-matching-tag))
-	((and (preceding-char) (or (= ?\) (char-syntax (preceding-char)))
-				   (= ?\" (preceding-char))))
-	 (backward-sexp))
-	((and (following-char) (or (= ?\( (char-syntax (following-char)))
-				   (= ?\" (following-char))))
-	 (forward-sexp))))
+  (if (memq major-mode hui-select-markup-modes)
+      (hui-select-goto-matching-tag)
+    (let* ((key (hypb:cmd-key-vector #'hui-select-goto-matching-delimiter
+				     hyperbole-mode-map))
+	   (major-mode-binding (lookup-key (current-local-map) key))
+	   (this-key-flag (and (called-interactively-p 'any)
+			       (equal (this-single-command-keys) key))))
+      (cond ((and major-mode-binding (not (integerp major-mode-binding))
+		  this-key-flag)
+	     ;; If the key that invokes this command in `hyperbole-minor-mode'
+	     ;; is also bound in the current major mode map, then
+	     ;; interactively invoke that command instead.  Typically
+	     ;; prevents clashes over {C-c .}.
+	     (call-interactively major-mode-binding))
+	    ((and (preceding-char) (or (= ?\) (char-syntax (preceding-char)))
+				       (= ?\" (preceding-char))))
+	     (backward-sexp))
+	    ((and (following-char) (or (= ?\( (char-syntax (following-char)))
+				       (= ?\" (following-char))))
+	     (forward-sexp))))))
 
 ;;;###autoload
 (defun hui-select-initialize ()
@@ -249,18 +390,17 @@ Also, add language-specific syntax setups to aid in thing selection."
     (require 'hyperbole))
   (when hkey-init
     (transient-mark-mode 1)
-    (hkey-global-set-key [double-down-mouse-1] nil)
-    (hkey-global-set-key [double-mouse-1] 'hui-select-thing-with-mouse)
-    (hkey-global-set-key [triple-down-mouse-1] nil)
-    (hkey-global-set-key [triple-mouse-1] 'hui-select-thing-with-mouse))
+    (hkey-set-key [double-down-mouse-1] nil)
+    (hkey-set-key [double-mouse-1] 'hui-select-thing-with-mouse)
+    (hkey-set-key [triple-down-mouse-1] nil)
+    (hkey-set-key [triple-mouse-1] 'hui-select-thing-with-mouse))
   ;;
   ;; These hooks let you select C++ and Java methods and classes by
   ;; double-clicking on the first character of a definition or on its
   ;; opening or closing brace.  This is all necessary since some
   ;; programmers don't put their function braces in the first column.
   (var:add-and-run-hook 'java-mode-hook (lambda ()
-					  (setq defun-prompt-regexp
-						"^[ \t]*\\(\\(\\(public\\|protected\\|private\\|const\\|abstract\\|synchronized\\|final\\|static\\|threadsafe\\|transient\\|native\\|volatile\\)\\s-+\\)*\\(\\(\\([[a-zA-Z][][_$.a-zA-Z0-9]*[][_$.a-zA-Z0-9]+\\|[[a-zA-Z]\\)\\s-*\\)\\s-+\\)\\)?\\(\\([[a-zA-Z][][_$.a-zA-Z0-9]*\\s-+\\)\\s-*\\)?\\([_a-zA-Z][^][ \t:;.,{}()=]*\\|\\([_$a-zA-Z][_$.a-zA-Z0-9]*\\)\\)\\s-*\\(([^);{}]*)\\)?\\([] \t]*\\)\\(\\s-*\\<throws\\>\\s-*\\(\\([_$a-zA-Z][_$.a-zA-Z0-9]*\\)[, \t\n\r\f]*\\)+\\)?\\s-*")))
+					  (setq defun-prompt-regexp hui-java-defun-prompt-regexp)))
   (var:add-and-run-hook 'c++-mode-hook (lambda ()
 					 (setq defun-prompt-regexp
 					       "^[ \t]*\\(template\\s-*<[^>;.{}]+>\\s-*\\)?\\(\\(\\(auto\\|const\\|explicit\\|extern\\s-+\"[^\"]+\"\\|extern\\|friend\\|inline\\|mutable\\|overload\\|register\\|static\\|typedef\\|virtual\\)\\s-+\\)*\\(\\([[<a-zA-Z][]_a-zA-Z0-9]*\\(::[]_a-zA-Z0-9]+\\)?\\s-*<[_<>a-zA-Z0-9 ,]+>\\s-*[*&]*\\|[[<a-zA-Z][]_<>a-zA-Z0-9]*\\(::[[<a-zA-Z][]_<>a-zA-Z0-9]+\\)?\\s-*[*&]*\\)[*& \t\n\r]+\\)\\)?\\(\\(::\\|[[<a-zA-Z][]_a-zA-Z0-9]*\\s-*<[^>;{}]+>\\s-*[*&]*::\\|[[<a-zA-Z][]_~<>a-zA-Z0-9]*\\s-*[*&]*::\\)\\s-*\\)?\\(operator\\s-*[^ \t\n\r:;.,?~{}]+\\(\\s-*\\[\\]\\)?\\|[_~<a-zA-Z][^][ \t:;.,~{}()]*\\|[*&]?\\([_~<a-zA-Z][_a-zA-Z0-9]*\\s-*<[^>;{}]+[ \t\n\r>]*>\\|[_~<a-zA-Z][_~<>a-zA-Z0-9]*\\)\\)\\s-*\\(([^{;]*)\\(\\(\\s-+const\\|\\s-+mutable\\)?\\(\\s-*[=:][^;{]+\\)?\\)?\\)\\s-*")))
@@ -271,15 +411,18 @@ Also, add language-specific syntax setups to aid in thing selection."
   ;; Allow for marking and moving brace delimited groups.
   (var:add-and-run-hook 'lisp-mode-hook
 			(lambda ()
-			  (modify-syntax-entry ?\{ "\(\}" lisp-mode-syntax-table)
-			  (modify-syntax-entry ?\} "\)\{" lisp-mode-syntax-table)))
+			  (modify-syntax-entry ?\{ "(}" lisp-mode-syntax-table)
+			  (modify-syntax-entry ?\} "){" lisp-mode-syntax-table)))
   ;;
   ;; This hook makes tags, comments, sentences and text blocks
   ;; selectable in SGML-related modes.
   ;;
   ;; Make tag begin and end delimiters act like grouping characters,
   ;; for easy syntactical selection of tags.
-  (let (hook-sym syntax-table keymap mode-str)
+  (let (hook-sym
+	mode-str
+	syntax-table-sym
+	keymap-sym)
     (mapc (lambda (mode)
             (setq mode-str (symbol-name mode)
                   hook-sym (intern (concat mode-str "-hook"))
@@ -289,10 +432,10 @@ Also, add language-specific syntax setups to aid in thing selection."
 			          `(lambda ()
                                      (let ((syntax-table (symbol-value ',syntax-table-sym))
                                            (keymap (symbol-value ',keymap-sym)))
-			               (modify-syntax-entry ?<  "(>" syntax-table)
-			               (modify-syntax-entry ?>  ")<" syntax-table)
-			               (modify-syntax-entry ?\{ "\(\}" syntax-table)
-			               (modify-syntax-entry ?\} "\)\{" syntax-table)
+			               ;; (modify-syntax-entry ?\< "(>" syntax-table)
+			               ;; (modify-syntax-entry ?\> ")<" syntax-table)
+			               (modify-syntax-entry ?\{ "(}" syntax-table)
+			               (modify-syntax-entry ?\} "){" syntax-table)
 			               (modify-syntax-entry ?\" "\"" syntax-table)
 			               (modify-syntax-entry ?=  "."  syntax-table)
 			               (modify-syntax-entry ?.  "_"  syntax-table)
@@ -314,42 +457,63 @@ Also, add language-specific syntax setups to aid in thing selection."
 	(hui-select-word (point)))))
 
 ;;;###autoload
-(defun hui-select-get-region ()
-  "Return the region that `hui-select-thing' would select."
+(defun hui-select-get-thing ()
+  "Return the thing at point that `hui-select-thing' would select."
   (let ((region-bounds (hui-select-get-region-boundaries)))
     (when region-bounds
       (buffer-substring-no-properties (car region-bounds) (cdr region-bounds)))))
 
+(defun hui-select-scan-sexps (from count)
+  "Scan FROM point across COUNT sexpressions."
+  (with-syntax-table hui-select-syntax-table
+    (scan-sexps from count)))
+
 ;;;###autoload
 (defun hui-select-thing ()
   "Select a region based on the syntax of the thing at point.
 If invoked repeatedly, this selects bigger and bigger things.
 If `hui-select-display-type' is non-nil and this is called
-interactively, the type of selection is displayed in the minibuffer."
+interactively, the type of selection is displayed in the minibuffer.
+
+If the key that invokes this command in `hyperbole-minor-mode' is
+also bound in the current major mode map, then interactively
+invoke that command instead.  Typically prevents clashes over
+{C-c RET}, {C-c C-m}."
   (interactive
    (cond ((and (fboundp 'use-region-p) (use-region-p))
 	  nil)
-	 ((and (boundp 'transient-mark-mode) transient-mark-mode mark-active)
+	 ((and transient-mark-mode mark-active)
 	  nil)
 	 (t
 	  ;; Reset selection based on the syntax of character at point.
 	  (hui-select-reset)
 	  nil)))
-  (let ((region (hui-select-get-region-boundaries)))
-    (unless region
-      (when (eq hui-select-previous 'punctuation)
-	(setq region (hui-select-word (point)))))
-    (when region
-      (goto-char (car region))
-      (set-mark (cdr region))
-      (when (fboundp 'activate-region) (activate-region))
-      (when (and (boundp 'transient-mark-mode)
-	         transient-mark-mode)
-	(setq mark-active t))
-      (and (called-interactively-p 'interactive) hui-select-display-type
-	   (message "%s" hui-select-previous))
-      (run-hooks 'hui-select-thing-hook)
-      t)))
+  (let* ((key (hypb:cmd-key-vector #'hui-select-thing hyperbole-mode-map))
+	 (major-mode-binding (lookup-key (current-local-map) key))
+	 (this-key-flag (and (called-interactively-p 'any)
+			     (equal (this-single-command-keys) key))))
+    (cond ((and major-mode-binding (not (integerp major-mode-binding))
+		this-key-flag)
+	   ;; If the key that invokes this command in `hyperbole-minor-mode'
+	   ;; is also bound in the current major mode map, then
+	   ;; interactively invoke that command instead.  Typically
+	   ;; prevents clashes over {C-c RET}, {C-c C-m}.
+	   (call-interactively major-mode-binding))
+	  ;;
+	  ;; No key conflicts, perform normal Hyperbole operation
+	  (t (let ((region (hui-select-get-region-boundaries)))
+	       (unless region
+		 (when (eq hui-select-previous 'punctuation)
+		   (setq region (hui-select-word (point)))))
+	       (when region
+		 (goto-char (car region))
+		 (set-mark (cdr region))
+		 (when transient-mark-mode
+		   (activate-mark))
+		 (and (called-interactively-p 'interactive) hui-select-display-type
+		      (message "%s" hui-select-previous))
+		 (run-hooks 'hui-select-thing-hook)
+		 t))))))
 
 ;;;###autoload
 (defun hui-select-thing-with-mouse (event)
@@ -376,10 +540,12 @@ displayed in the minibuffer."
 
 ;;;###autoload
 (defun hui-select-goto-matching-tag ()
-  "If in a major mode listed in `hui-select-markup-modes,' move point to the start of the tag paired with the closest tag that point is within or precedes.
-Returns t if point is moved, else nil.
-Signals an error if no tag is found following point or if the closing tag
-does not have a `>' terminator character."
+  "Move point to start of the tag paired with closest tag point is at or precedes.
+Tag in this context is an sgml-like tag surrounded by angle brackets, <>.
+Enabled in major modes in `hui-select-markup-modes'.  Returns t if
+point is moved, else nil.  Signals an error if no tag is found
+following point or if the closing tag does not have a `>'
+terminator character."
   (interactive)
   (when (memq major-mode hui-select-markup-modes)
     (let ((result)
@@ -481,7 +647,7 @@ does not have a `>' terminator character."
 
 ;;;###autoload
 (defun hui-select-and-copy-thing ()
-  "Copy the region surrounding the syntactical unit at point."
+  "Copy the region surrounding the syntactical unit at point to the kill ring."
   (interactive)
   (let ((bounds (hui-select-boundaries (point))))
     (when bounds
@@ -501,7 +667,8 @@ does not have a `>' terminator character."
 ;;
 
 (defun hui-select-boundaries (pos)
-  "Return the (start . end) of a syntactically defined region based upon the last region selected or on position POS.
+  "Return the (start . end) of a syntactically defined region.
+This is based upon the last region selected or on position POS.
 The character at POS is selected if no other thing is matched."
   (interactive)
   (setcar hui-select-old-region (car hui-select-region))
@@ -561,7 +728,7 @@ The character at POS is selected if no other thing is matched."
 
 ;;;###autoload
 (defun hui-select-double-click-hook (event click-count)
-  "Select a region based on the syntax of the character wherever the mouse is double-clicked.
+  "Select region based on the character syntax where the mouse is double-clicked.
 If the double-click occurs at the same point as the last double-click, select
 the next larger syntactic structure.  If `hui-select-display-type' is non-nil,
 the type of selection is displayed in the minibuffer."
@@ -571,7 +738,7 @@ the type of selection is displayed in the minibuffer."
 	(t (hui-select-thing-with-mouse event))))
 
 (defun hui-select-syntactical-region (pos)
-  "Return the (start . end) of a syntactically defined region based upon the buffer position POS.
+  "Return the (start . end) of a syntactically defined region based upon POS.
 Uses `hui-select-syntax-alist' and the current buffer's syntax table to
 determine syntax groups.
 
@@ -603,8 +770,10 @@ If an error occurs during syntax scanning, it returns nil."
 	     (hui-select-set-region pos (1+ pos)))))))
 
 (defun hui-select-at-delimited-thing-p ()
-  "Return non-nil if point is at a markup pair, list, array/vector, set, comment or string, else nil.
-The non-nil value returned is the function to call to select that syntactic unit.
+  "Return non-nil if point is at a delimited thing, else nil.
+A delimited tings is a markup pair, list, array/vector, set,
+comment or string.  The non-nil value returned is the function to
+call to select that syntactic unit.
 
 Ignores any match if on an Emacs button and instead returns nil."
   (unless (button-at (point))
@@ -620,7 +789,8 @@ Ignores any match if on an Emacs button and instead returns nil."
 	  (t hkey-value))))
 
 (defun hui-select-delimited-thing ()
-  "Select a markup pair, list, array/vector, set, comment or string at point and return t, else nil."
+  "Select a markup pair, list, array/vector, set, comment or string at point.
+Return t is selected, else nil."
   (interactive)
   (prog1 (and (hui-select-delimited-thing-call #'hui-select-thing) t)
     ;; If selected region is followed by only whitespace and then a
@@ -644,29 +814,28 @@ Ignores any match if on an Emacs button and instead returns nil."
 	       (/= ?\\ (char-syntax (char-before (1- (point))))))))))
 
 (defun hui-select-mark-delimited-sexp ()
-  "If point is before or after an sexp and not at an end of line, then deactivate the mark and mark the sexp.
-Return t if marked, nil otherwise.  If any error occurs such as
-  unbalanced start and end sexp delimiters, ignore it, and return
-  nil."
+  "When point is before or after an sexp deactivate the mark and mark the sexp.
+Not enabled when point is at an end of line.  Return t if marked,
+nil otherwise.  If any error occurs such as unbalanced start and
+end sexp delimiters, ignore it, and return nil."
   (interactive)
   (let ((mark-sexp-func (lambda ()
 			  (when (region-active-p) (deactivate-mark))
 			  (mark-sexp) t)))
-    (condition-case nil
-	(let ((syn-after (char-syntax (char-after)))
-	      syn-before)
-	  (cond ((eq syn-after ?\()
-		 (funcall mark-sexp-func))
-		((eq syn-after ?\))
-		 (forward-char 1)
-		 (backward-sexp)
-		 (funcall mark-sexp-func))
-		((and (not (eolp))
-		      (setq syn-before (char-syntax (char-before)))
-		      (eq syn-before ?\)))
-		 (backward-sexp)
-		 (funcall mark-sexp-func))))
-      (error nil))))
+    (ignore-errors
+      (let ((syn-after (char-syntax (char-after)))
+	    syn-before)
+	(cond ((eq syn-after ?\()
+	       (funcall mark-sexp-func))
+	      ((eq syn-after ?\))
+	       (forward-char 1)
+	       (backward-sexp)
+	       (funcall mark-sexp-func))
+	      ((and (not (eolp))
+		    (setq syn-before (char-syntax (char-before)))
+		    (eq syn-before ?\)))
+	       (backward-sexp)
+	       (funcall mark-sexp-func)))))))
 
 ;;; ************************************************************************
 ;;; Private functions
@@ -674,7 +843,7 @@ Return t if marked, nil otherwise.  If any error occurs such as
 
 (defun hui-select-at-blank-line-or-comment ()
   "Return non-nil if on a blank line or a comment start or end line.
-Assumes point is before any non-whitespace character on the line."
+Assume point is before any non-whitespace character on the line."
   (let ((comment-end-p (and (stringp comment-end)
 			    (not (string-equal comment-end "")))))
     (if (looking-at
@@ -699,7 +868,7 @@ Assumes point is before any non-whitespace character on the line."
 This respects the current syntax table definition of whitespace, whereas
 `back-to-indentation' does not.  This is relevant in literate programming and
 mail and news reply modes."
-  (goto-char (min (progn (end-of-line) (point))
+  (goto-char (min (line-end-position)
 		  (progn (beginning-of-line)
 			 (skip-syntax-forward " ")
 			 (point)))))
@@ -711,8 +880,10 @@ mail and news reply modes."
     (setq this-command 'select-thing)))
 
 (defun hui-select-delimited-thing-call (func)
-  "Select a markup pair, list, vector/array, set, comment or string at point and return non-nil, else nil.
-The non-nil value returned is the function to call to select that syntactic unit."
+  "Select a delimited thing at point and return a function to select it.
+The delimited thing is a markup pair, list, vector/array, set,
+comment or string.  The non-nil value returned is the function to
+call to select that syntactic unit."
   (unless (and (memq major-mode hui-select-ignore-quoted-sexp-modes)
 	       ;; Ignore quoted identifier sexpressions, like #'function
 	       (char-after) (memq (char-after) '(?# ?\')))
@@ -731,7 +902,9 @@ The non-nil value returned is the function to call to select that syntactic unit
       (funcall func))))
 
 (defun hui-select-region-bigger-p (old-region new-region)
-  "Return t if OLD-REGION is smaller than NEW-REGION and NEW-REGION partially overlaps OLD-REGION, or if OLD-REGION is uninitialized."
+  "Non-nil means the new region is bigger than the old region.
+Return t if OLD-REGION is smaller than NEW-REGION and NEW-REGION
+partially overlaps OLD-REGION, or if OLD-REGION is uninitialized."
   (if (null (car old-region))
       t
     (and (> (abs (- (cdr new-region) (car new-region)))
@@ -770,7 +943,7 @@ Return the updated cons cell."
     hui-select-region))
 
 (defun hui-select-string-p (&optional start-delim end-delim)
-  "Return (start . end) of string whose first line point is within or immediately before.
+  "Return (start . end) of string whose first line point is in or directly before.
 Positions include delimiters.  String is delimited by double quotes unless
 optional START-DELIM and END-DELIM (strings) are given.
 Returns nil if not within a string."
@@ -817,9 +990,11 @@ Returns nil if not within a string."
 ;;;
 
 (defun hui-select-brace-def-or-declaration (pos)
-  "If POS is at the first character, opening brace or closing brace of a brace delimited language definition, return (start . end) region, else nil.
-The major mode for each supported brace language must be included in the
-list, hui-select-brace-modes."
+  "Return (start . end) region of a brace delimited language definition at POS.
+If POS is at the first character, opening brace or closing brace
+of a brace delimited language definition, return (start . end)
+region, else nil.  The major mode for each supported brace
+language must be included in the list, hui-select-brace-modes."
   (interactive)
   (when (and (featurep 'cc-mode) (memq major-mode hui-select-brace-modes))
     (save-excursion
@@ -836,16 +1011,14 @@ list, hui-select-brace-modes."
 			(looking-at defun-prompt-regexp)))
 		 (and (= (following-char) ?\})
 		      (stringp defun-prompt-regexp)
-		      (condition-case ()
-			  (progn
-			    ;; Leave point at opening brace.
-			    (goto-char
-			     (scan-sexps (1+ (point)) -1))
-			    ;; Test if these are defun braces.
-			    (save-excursion
-			      (beginning-of-line)
-			      (looking-at defun-prompt-regexp)))
-			(error nil)))))
+		      (ignore-errors
+			;; Leave point at opening brace.
+			(goto-char
+			 (hui-select-scan-sexps (1+ (point)) -1))
+			;; Test if these are defun braces.
+			(save-excursion
+			  (beginning-of-line)
+			  (looking-at defun-prompt-regexp))))))
 	    eod)
 	(when (or at-def-brace
 		  ;; At the start of a definition:
@@ -907,7 +1080,7 @@ list, hui-select-brace-modes."
 			  (error (point-max)))))
 	    (when (= (following-char) ?\})
 	      ;; Leave point at opening brace.
-	      (goto-char (scan-sexps (1+ (point)) -1)))
+	      (goto-char (hui-select-scan-sexps (1+ (point)) -1)))
 	    (when (= (following-char) ?\{)
 	      (while (and (zerop (forward-line -1))
 			  (not (hui-select-at-blank-line-or-comment))))
@@ -979,13 +1152,13 @@ list, hui-select-indent-modes."
       ;; an outline entry, in which case we mark entries as indented blocks.
       (hui-select-indent-def pos)
       (save-excursion
-        (when (memq (char-syntax (if (eobp) (preceding-char) (char-after pos)))
+        (when (memq (char-syntax (if (smart-eobp) (preceding-char) (char-after pos)))
 	          '(?w ?_))
 	  (setq hui-select-previous 'symbol)
 	  (condition-case ()
-	      (let ((end (scan-sexps pos 1)))
+	      (let ((end (hui-select-scan-sexps pos 1)))
 		(hui-select-set-region
-		 (min pos (scan-sexps end -1)) end))
+		 (min pos (hui-select-scan-sexps end -1)) end))
 	    (error nil))))))
 
 (defun hui-select-sexp-start (pos)
@@ -994,28 +1167,23 @@ list, hui-select-indent-modes."
       (hui-select-brace-def-or-declaration pos)
       (save-excursion
 	(setq hui-select-previous 'sexp-start)
-	(condition-case ()
-	    (hui-select-set-region pos (scan-sexps pos 1))
-	  (error nil)))))
+	(ignore-errors (hui-select-set-region pos (hui-select-scan-sexps pos 1))))))
 
 (defun hui-select-sexp-end (pos)
   "Return (start . end) of sexp ending at POS."
   (or (hui-select-brace-def-or-declaration pos)
       (save-excursion
 	(setq hui-select-previous 'sexp-end)
-	(condition-case ()
-	    (hui-select-set-region (scan-sexps (1+ pos) -1) (1+ pos))
-	  (error nil)))))
+	(ignore-errors (hui-select-set-region (hui-select-scan-sexps (1+ pos) -1) (1+ pos))))))
 
 (defun hui-select-sexp (pos)
   "Return (start . end) of the sexp that POS is within."
   (setq hui-select-previous 'sexp)
   (save-excursion
     (goto-char pos)
-    (condition-case ()
-	(hui-select-set-region (progn (backward-up-list 1) (point))
-			       (progn (forward-list 1) (point)))
-      (error nil))))
+    (ignore-errors
+      (hui-select-set-region (progn (backward-up-list 1) (point))
+			     (progn (forward-list 1) (point))))))
 
 (defun hui-select-sexp-up (pos)
   "Return (start . end) of the sexp enclosing the selected area or nil."
@@ -1025,10 +1193,9 @@ list, hui-select-indent-modes."
   (setq pos (or (car hui-select-region) pos))
   (save-excursion
     (goto-char pos)
-    (condition-case ()
-	(hui-select-set-region (progn (backward-up-list 1) (point))
-			       (progn (forward-list 1) (point)))
-      (error nil))))
+    (ignore-errors
+      (hui-select-set-region (progn (backward-up-list 1) (point))
+			     (progn (forward-list 1) (point))))))
 
 (defun hui-select-preprocessor-def (pos)
   "Return (start . end) of a preprocessor #definition starting at POS, if any.
@@ -1056,7 +1223,8 @@ included in the list, hui-select-brace-modes."
 ;; the previous and subsequent sexpression.  Useful in contexts such as
 ;; 'foo.bar'.
 (defun hui-select-punctuation (pos)
-  "Return (start . end) region including sexpressions before and after POS, when at a punctuation character."
+  "Return (start . end) region when at a punctuation character.
+The region includes sexpressions before and after POS"
   (or (hui-select-comment pos)
       (hui-select-preprocessor-def pos)
       (hui-select-brace-def-or-declaration pos) ;; Might be on a C++ destructor ~.
@@ -1068,11 +1236,9 @@ included in the list, hui-select-brace-modes."
 	      ((and (char-before pos) (= ?\  (char-syntax (char-before pos))))
 	       (hui-select-set-region (1- pos) pos))
 	      (t (goto-char pos)
-		 (condition-case ()
-		     (hui-select-set-region
-		      (save-excursion (backward-sexp) (point))
-		      (progn (forward-sexp) (point)))
-		   (error nil)))))))
+		 (ignore-errors (hui-select-set-region
+				 (save-excursion (backward-sexp) (point))
+				 (progn (forward-sexp) (point)))))))))
 
 (defun hui-select-comment (pos)
   "Return rest of line from POS to newline."
@@ -1187,15 +1353,14 @@ Delimiters may be single, double or open and close quotes."
   (setq hui-select-previous 'sentence)
   (save-excursion
     (goto-char pos)
-    (condition-case ()
-	(hui-select-set-region (progn (backward-sentence) (point))
-			       (progn (forward-sentence) (point)))
-      (error nil))))
+    (ignore-errors
+      (hui-select-set-region (progn (backward-sentence) (point))
+			     (progn (forward-sentence) (point))))))
 
 (defun hui-select-whitespace (pos)
-  "Return (start . end) of all but one char of whitespace POS, unless
-there is only one character of whitespace or this is leading whitespace on
-the line.  Then return all of it."
+  "Return (start . end) of all whitespace at POS,
+Return all whitespace, unless there is only one character of
+whitespace or this is leading whitespace on the line."
   (setq hui-select-previous 'whitespace)
   (save-excursion
     (goto-char pos)
@@ -1209,9 +1374,11 @@ the line.  Then return all of it."
 	    (hui-select-set-region (1+ start) end)))))))
 
 (defun hui-select-markup-pair (pos)
-  "Return (start . end) of region between the opening and closing of an HTML, XML or SGML tag pair, one of which is at POS.
-The major mode for each language that uses such tags must be included in the
-list, hui-select-markup-modes."
+  "Return (start . end) of region of markup pair, one of which is at POS.
+The region returned is between the opening and closing tag.
+Markups recognised are HTML, XML or SGML tag pairs.  The major
+mode for each language that uses such tags must be included in
+the list, hui-select-markup-modes."
   (when (memq major-mode hui-select-markup-modes)
     (setq hui-select-previous 'markup-pair)
     (let ((pos-with-space)
@@ -1250,7 +1417,7 @@ list, hui-select-markup-modes."
 			  (throw 'done opoint))
 		        (setq count (1- count))
 		        (when (= count 0)
-                          (cond ((eobp)
+                          (cond ((smart-eobp)
                                  (if bolp
 				     ;; Include leading space since the
 				     ;; start and end tags begin and end
@@ -1315,7 +1482,8 @@ list, hui-select-markup-modes."
 ;;;
 
 (defun hui-select-line (pos)
-  "Return (start . end) of the whole line POS is in, with newline unless at end of buffer."
+  "Return (start . end) of the whole line POS is in.
+Includes newline unless at end of buffer."
   (setq hui-select-previous 'line)
   (save-excursion
     (goto-char pos)
@@ -1341,81 +1509,11 @@ list, hui-select-markup-modes."
     (hui-select-set-region (progn (backward-page) (point))
 			   (progn (forward-page) (point)))))
 
-(defun hui-select-buffer (pos)
+(defun hui-select-buffer (_pos)
   "Return (start . end) of the buffer at POS."
   (setq hui-select-previous 'buffer)
   (hui-select-set-region (point-min) (point-max)))
 
-;;; ************************************************************************
-;;; Private variables
-;;; ************************************************************************
-
-(defconst hui-select-syntax-table (make-syntax-table emacs-lisp-mode-syntax-table)
-  "Syntax table to use when selecting delimited things.")
-;; Make braces be thing delimiters, not punctuation.
-(modify-syntax-entry ?\{ "\(\}" hui-select-syntax-table)
-(modify-syntax-entry ?\} "\)\{" hui-select-syntax-table)
-
-(defvar hui-select-bigger-alist
-  '((char nil)
-    (whitespace hui-select-whitespace)
-    (word hui-select-word)
-    (symbol hui-select-symbol)
-    (punctuation nil)
-    (string hui-select-string)
-    (text nil)
-    (comment hui-select-comment)
-    (markup-pair nil)
-    (preprocessor-def nil)
-    (sexp hui-select-sexp)
-    (sexp-start nil)
-    (sexp-end nil)
-    (sexp-up hui-select-sexp-up)
-    (line hui-select-line)
-    (sentence hui-select-sentence)
-    (brace-def-or-declaration hui-select-brace-def-or-declaration)
-    (indent-def hui-select-indent-def)
-    (paragraph hui-select-paragraph)
-    (page hui-select-page)
-    (buffer hui-select-buffer)
-    )
-  "Unordered list of (<region-type-symbol> <region-selection-function>) pairs.
-Used to go from one thing to a bigger thing.  See `hui-select-bigger-thing'.
-Nil value for <region-selection-function> means that region type is skipped
-over when trying to grow the region and is only used when a selection is made
-with point on a character that triggers that type of selection.")
-
-(defvar hui-select-prior-buffer nil)
-(defvar hui-select-prior-point nil)
-
-(defvar hui-select-previous 'char
-  "Most recent type of selection.  Must be set by all hui-select functions.")
-
-(defvar hui-select-region (cons nil nil)
-  "Cons cell that contains a region (<beginning> . <end>).
-The function `hui-select-set-region' updates and returns it.")
-
-(defvar hui-select-old-region (cons nil nil)
-  "Cons cell that contains a region (<beginning> . <end>).")
-
-(defcustom hui-select-syntax-alist
-  '((?w  . hui-select-word)
-    (?_  . hui-select-symbol)
-    (?\" . hui-select-string)
-    (?\( . hui-select-sexp-start)
-    (?\$ . hui-select-sexp-start)
-    (?\' . hui-select-sexp-start)
-    (?\) . hui-select-sexp-end)
-    (?   . hui-select-whitespace)
-    (?\< . hui-select-comment)
-    (?\. . hui-select-punctuation))
-  "*Unordered list of pairs of the form (<syntax-char> <function>) used by the function `hui-select-syntactical-region'.
-Each <function> takes a single position argument and returns a
-region (start . end) defining the boundaries of the thing at that position."
-  :type '(repeat (cons (character :tag "Syntax-Char") function))
-  :group 'hyperbole-commands)
-
-
 (provide 'hui-select)
 
 ;;; hui-select.el ends here
