@@ -1,11 +1,14 @@
-;;; hversion.el --- GNU Hyperbole version and system information setup
+;;; hversion.el --- GNU Hyperbole version and system information setup  -*- lexical-binding: t; -*-
 ;;
 ;; Author:       Bob Weiner
 ;; Maintainer:   Bob Weiner, Mats Lidell
 ;;
 ;; Orig-Date:     1-Jan-94
+;; Last-Mod:      3-Dec-23 at 09:43:50 by Bob Weiner
 ;;
-;; Copyright (C) 1994-2020  Free Software Foundation, Inc.
+;; SPDX-License-Identifier: GPL-3.0-or-later
+;;
+;; Copyright (C) 1994-2022  Free Software Foundation, Inc.
 ;; See the "HY-COPY" file for license information.
 ;;
 ;; This file is part of GNU Hyperbole.
@@ -19,30 +22,28 @@
 
 (require 'hload-path)
 
+;;; ************************************************************************
+;;; Public declarations
+;;; ************************************************************************
+
+(declare-function br-to-view-window "ext:br")
+(declare-function hpath:substitute-value "hpath")
+(declare-function id-info-item "hversion")
+(declare-function br-in-browser "hpath")
+
 ;;; ************************************************************************
 ;;; Public variables
 ;;; ************************************************************************
 
-(defconst hyperb:version "7.1.3" "GNU Hyperbole revision number.")
+(defconst hyperb:version "9.0.0" "GNU Hyperbole revision number.")
 
-;;;###autoload
-(defvar hyperb:microsoft-os-p
-  (memq system-type '(ms-windows windows-nt ms-dos win32))
-  "Non-nil iff Hyperbole is running under a Microsoft OS but not under Windows Subsystem for Linux (WSL).
-Use `hyperb:wsl-os-p' to test if running under WSL.")
-
-;;;###autoload
-(defvar hyperb:wsl-os-p
-  (and (eq system-type 'gnu/linux) (executable-find "wsl.exe") t)
-  "T iff Hyperbole is running under Microsoft Windows Subsystem for Linux (WSL).")
-
-;;;###autoload
 (defvar hyperb:mouse-buttons
   (if (or (and hyperb:microsoft-os-p (not (memq window-system '(w32 w64 x))))
-	  (memq window-system '(ns dps)))
+	  (memq window-system '(mac ns dps)))
       2 3)
   "Number of live buttons available on the mouse.
-Override this if the system-computed default is incorrect for your specific mouse.")
+Override this if the system-computed default is incorrect for
+your specific mouse.")
 
 (defvar hyperb:automount-prefixes
   (if (and (boundp 'automount-dir-prefix) (stringp automount-dir-prefix))
@@ -56,9 +57,10 @@ Override this if the system-computed default is incorrect for your specific mous
 
 ;; Called in hyperbole.el.
 (defun hyperb:stack-frame (function-list &optional debug-flag)
-  "Return the nearest Emacs Lisp stack frame which called any function symbol from FUNCTION-LIST or nil if no match.
-If FUNCTION-LIST contains 'load, 'autoload or 'require, detect
-autoloads not visible within the Lisp level stack frames.
+  "Return the nearest Elisp stack frame that called a function from FUNCTION-LIST.
+Return nil if there is no match.  FUNCTION-LIST entries must be symbols.
+If FUNCTION-LIST contains \\='load, \\='autoload or \\='require, detect autoloads
+not visible within the Lisp level stack frames.
 
 With optional DEBUG-FLAG non-nil, if no matching frame is found, return list
 of stack frames (from innermost to outermost)."
@@ -72,7 +74,7 @@ of stack frames (from innermost to outermost)."
 	frame)
     (or (catch 'hyperb:stack-frame
 	  (while (setq frame (backtrace-frame count))
-	    (if debug-flag (setq frame-list (cons frame frame-list)))
+	    (when debug-flag (setq frame-list (cons frame frame-list)))
 	    (setq count (1+ count)
 		  fsymbol (nth 1 frame))
 	    (and (eq fsymbol 'command-execute)
@@ -92,40 +94,13 @@ of stack frames (from innermost to outermost)."
 		  ((memq fsymbol function-list)
 		   (throw 'hyperb:stack-frame frame))))
 	  nil)
-	(if debug-flag (nreverse frame-list)))))
-
-(defun hyperb:path-being-loaded ()
-  "Return the full pathname used by the innermost `load' or 'require' call.
-Removes any matches for `hyperb:automount-prefixes' before returning
-the pathname."
-  (let* ((frame (hyperb:stack-frame '(load require)))
-	 (function (nth 1 frame))
-	 file nosuffix)
-    (cond ((eq function 'load)
-	   (setq file (nth 2 frame)
-		 nosuffix (nth 5 frame)))
-	  ((eq function 'require)
-	   (setq file (or (nth 3 frame) (symbol-name (nth 2 frame))))))
-    (if (stringp file)
-	(setq nosuffix (or nosuffix
-			   (string-match
-			    "\\.\\(elc?\\|elc?\\.gz\\|elc?\\.Z\\)$"
-			    file))
-	      file (substitute-in-file-name file)
-	      file (locate-file file load-path
-				(if (null nosuffix) '(".elc" ".el" ".el.gz" ".el.Z"))
-				;; accept any existing file
-				nil)
-	      file (if (and (stringp file)
-			    (string-match hyperb:automount-prefixes file))
-		       (substring file (1- (match-end 0)))
-		     file)))))
+	(when debug-flag (nreverse frame-list)))))
 
 (defun hyperb:window-sys-term (&optional frame)
-  "Return the first part of the term-type if running under a window system, else nil.
+  "Return first part of the term-type if running under a window system, else nil.
 Where a part in the term-type is delimited by a `-' or  an `_'."
   (unless frame (setq frame (selected-frame)))
-  (let* ((display-type (if (fboundp 'device-type) (device-type) window-system))
+  (let* ((display-type window-system)
 	 (term (cond ((or (memq display-type '(x gtk mswindows win32 w32 ns dps pm))
 			  ;; May be a graphical client spawned from a
 			  ;; dumb terminal Emacs, e.g. under X, so if
@@ -134,24 +109,24 @@ Where a part in the term-type is delimited by a `-' or  an `_'."
 			  (display-mouse-p))
 		      ;; X11, macOS, NEXTSTEP (DPS), or OS/2 Presentation Manager (PM)
 		      "emacs")
+		     ;; Keep NeXT as basis for 2-button mouse support
 		     ((or (featurep 'eterm-fns)
 			  (equal (getenv "TERM") "NeXT")
 			  (equal (getenv "TERM") "eterm"))
 		      ;; NEXTSTEP add-on support to Emacs
-		      "next")
-		     )))
-    (set-frame-parameter frame 'hyperb:window-system
-			 (and term (setq term (substring term 0 (string-match "[-_]" term)))))
+		      "next"))))
+    (set-frame-parameter frame 'hyperb:window-system term)
     term))
 
 (defun hyperb:window-system (&optional frame)
-  "Return the string name for window system or term type under which the selected FRAME is running.
-If nil after system initialization, no window system or mouse support is available."
+  "Return name of window system or term type where the selected FRAME is running.
+If nil after system initialization, no window system or mouse
+support is available."
   (unless frame (setq frame (selected-frame)))
   (frame-parameter frame 'hyperb:window-system))
 
 ;; Each frame could be on a different window system when under a
-;; client-server window system, so set `hyperb:window-system'  for
+;; client-server window system, so set `hyperb:window-system' for
 ;; each frame.
 (mapc #'hyperb:window-sys-term (frame-list))
 ;; Ensure this next hook is appended so that if follows the hook that
@@ -162,7 +137,7 @@ If nil after system initialization, no window system or mouse support is availab
 ;;; Public functions used by pulldown and popup menus
 ;;; ************************************************************************
 
-(if (not (fboundp 'id-browse-file))
+(unless (fboundp 'id-browse-file)
 (defalias 'id-browse-file 'view-file))
 
 (unless (fboundp 'id-info)
@@ -191,7 +166,7 @@ If nil after system initialization, no window system or mouse support is availab
 	     ;; Force execution of Info-mode-hook which adds the
 	     ;; Hyperbole man directory to Info-directory-list.
 	     (info)
-	     (if (string-match "^(\\([^\)]+\\))\\(.*\\)" index-item)
+	     (if (string-match "^(\\([^)]+\\))\\(.*\\)" index-item)
 		 (let ((file (match-string-no-properties 1 index-item))
 		       (item-name (match-string-no-properties 2 index-item)))
 		   (if (and file (setq file (hpath:substitute-value file)))
@@ -199,14 +174,20 @@ If nil after system initialization, no window system or mouse support is availab
 			      (Info-index item-name))
 		     (Info-goto-node "(hyperbole)")
 		     (Info-index index-item))
+		   ;; Index may point to indented line immediately
+		   ;; after the non-indented item definition line. If
+		   ;; so, move back a line.
+		   (when (and (looking-at "^[ \t]")
+			      (looking-back "^[^ \t].*[\n\r]+" nil))
+		     (forward-line -1))
 		   (recenter 0))
 	       (error "(id-info-item): Invalid Info index item: `%s'" index-item)))
     (error "(id-info-item): Info index item must be a string: `%s'" index-item))))
 
-(if (not (fboundp 'id-tool-quit))
-(defalias 'id-tool-quit 'eval))
+(unless (fboundp 'id-tool-quit)
+(defalias 'id-tool-quit #'eval))
 
-(if (not (fboundp 'id-tool-invoke))
+(unless (fboundp 'id-tool-invoke)
 (defun id-tool-invoke (sexp)
   (if (commandp sexp)
       (call-interactively sexp)
